package com.tofly.yxpc.base;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.util.DisplayMetrics;
import android.view.WindowManager;
import com.tofly.latte_core.config.Latte;
import com.tofly.latte_core.net.exception.ExceptionHandle;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.latte_core.utils.ActivityUtils;
import com.tofly.yxpc.BuildConfig;
import com.tofly.yxpc.greenDao.DaoMaster;
import com.tofly.yxpc.greenDao.DaoSession;
import com.tofly.yxpc.ui.activity.LoginActivity;
import com.tofly.yxpc.utils.db.UpgradeHelper;
import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;

/**
 * @author yck
 */
public class MyApp extends Application {

    private static MyApp instance = null;

    /**
     * 数据库对象
     */
    private DaoSession mDaoSession;

    /**
     * 当前设备分辨率
     */
    private static float density;
    private long lastClickTime = 0;

    @Override
    public void onCreate() {
        super.onCreate();
        instance = this;
        init();
        getDisplayParameters();
        EventBus.getDefault().register(this);
    }

    /**
     * 获得当前设备的屏幕参数
     */
    private void getDisplayParameters() {
        DisplayMetrics dm = new DisplayMetrics();
        WindowManager wm = (WindowManager) getSystemService(Context.WINDOW_SERVICE);
        wm.getDefaultDisplay().getMetrics(dm);
        density = dm.density;
    }

    private void init() {
        boolean isLog = false;
        if (BuildConfig.DEBUG) {
            isLog = true;
        }
        Latte.init(this)
                .withInterceptor(isLog)
                .withApiHost(NetWorkUrl.getServerUrl())
                .configure();

        initDataBase();
    }

    private void initDataBase() {
        UpgradeHelper mHelper = new UpgradeHelper(this, "yx_pc.db");
        SQLiteDatabase database = mHelper.getWritableDatabase();
        DaoMaster master = new DaoMaster(database);
        mDaoSession = master.newSession();
    }

    public static int dip2px(float dipValue) {
        final float scale = density;
        return (int) (dipValue * scale + 0.5f);
    }

    public DaoSession getDaoSession() {
        return mDaoSession;
    }

    public static MyApp getInstance() {
        return instance;
    }

    public boolean isOverTime() {
        boolean isSuccess = false;
        long currentTime = System.currentTimeMillis();
        long mClickTime = 1000;
        if (currentTime - lastClickTime > mClickTime) {
            lastClickTime = currentTime;
            isSuccess = true;
        }
        return isSuccess;
    }

    @Subscribe
    public void onMessageEvent(ExceptionHandle.MyExceptionEvent event) {
        if (ExceptionHandle.MyExceptionEvent.UNAUTHORIZED_EVENT.equals(event.message)) {
            Activity activity = ActivityUtils.getInstance().getCurrentActivity();
            if (activity != null && !"ui.activity.LoginActivity".equals(activity.getLocalClassName())) {
                AccountManager.setToken("");
                Intent intent = new Intent(activity, LoginActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
                intent.putExtra("isLoss", true);
                startActivity(intent);
                ActivityUtils.getInstance().finishAllActivity();
            }
        }
    }
}
