package com.tofly.yxpc.map.gaode;

import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.maps.model.LatLng;
import com.amap.api.services.core.LatLonPoint;
import com.amap.api.services.geocoder.GeocodeResult;
import com.amap.api.services.geocoder.GeocodeSearch;
import com.amap.api.services.geocoder.RegeocodeQuery;
import com.amap.api.services.geocoder.RegeocodeResult;
import com.tofly.latte_core.config.Latte;
import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.map.LocationUtil_xu;

/**
 * @author ychk
 */
public class AmapLocationHelper implements GeocodeSearch.OnGeocodeSearchListener {

    private AMapLocationClient locationClient;
    private GeocodeSearch geocodeSearch;
    protected LatLng arcgisPoint = null;
    protected String mAddress = null;
    private boolean isFirstLoc = true;
    private AmapReserveLocListener mReserveLocListener = null;
    private AmapLocChangeListener mLocationListener = null;
    private int num = 0;
    private int num1 = 0;

    public AmapLocationHelper() {
        init();
    }

    private void init() {
        initAmapGeocodeSearch();
        initAmapLocationService();
    }

    /**
     * 定位监听
     */
    public void resetLocation() {
        isFirstLoc = true;
    }

    /*定位服务*/
    private void initAmapLocationService() {
        locationClient = new AMapLocationClient(Latte.getApplicationContext());
        locationClient.setLocationOption(getDefaultOption());
        locationClient.setLocationListener(aMapLocationListener);
    }

    /**
     * 默认的定位参数
     *
     * @author hongming.wang
     * @since 2.8.0
     */
    private AMapLocationClientOption getDefaultOption() {
        AMapLocationClientOption mOption = new AMapLocationClientOption();
        mOption.setLocationMode(AMapLocationClientOption.AMapLocationMode.Hight_Accuracy);//可选，设置定位模式，可选的模式有高精度、仅设备、仅网络。默认为高精度模式
        mOption.setGpsFirst(true);//可选，设置是否gps优先，只在高精度模式下有效。默认关闭
        mOption.setHttpTimeOut(30000);//可选，设置网络请求超时时间。默认为30秒。在仅设备模式下无效
        mOption.setInterval(3000);//可选，设置定位间隔。默认为2秒
        mOption.setNeedAddress(true);//可选，设置是否返回逆地理地址信息。默认是true
        mOption.setOnceLocation(false);//可选，设置是否单次定位。默认是false
        mOption.setOnceLocationLatest(false);//可选，设置是否等待wifi刷新，默认为false.如果设置为true,会自动变为单次定位，持续定位时不要使用
        AMapLocationClientOption.setLocationProtocol(AMapLocationClientOption.AMapLocationProtocol.HTTP);//可选， 设置网络请求的协议。可选HTTP或者HTTPS。默认为HTTP
        mOption.setSensorEnable(false);//可选，设置是否使用传感器。默认是false
        mOption.setWifiScan(true); //可选，设置是否开启wifi扫描。默认为true，如果设置为false会同时停止主动刷新，停止以后完全依赖于系统刷新，定位位置可能存在误差
        mOption.setLocationCacheEnable(false); //可选，设置是否使用缓存定位，默认为true
        return mOption;
    }

    /*开始定位*/
    public void startLocating() {
        if (locationClient != null) {
            locationClient.startLocation();
        }
    }

    public interface AmapLocChangeListener {
        void onLocationed(LatLng location, String addr, LatLng longPoint);
    }

    public void setMapLocationListener(AmapLocChangeListener locationListener) {
        this.mLocationListener = locationListener;
    }

    private AMapLocationListener aMapLocationListener = location -> {
        if (null != location) {
            //errCode等于0代表定位成功，其他的为定位失败，具体的可以参照官网定位错误码说明
            if (location.getErrorCode() == 0) {
                if (isFirstLoc) {
                    isFirstLoc = false;
                    LocationUtil_xu.CoordinateClass coordinate = LocationUtil_xu.gcj02towgs84(location.getLongitude(), location.getLatitude());
                    if (coordinate == null) {
                        return;
                    }
                    double lng = LocationUtil_xu.formatLocationDouble(coordinate.Longtitude);
                    double lat = LocationUtil_xu.formatLocationDouble(coordinate.Latitude);
                    arcgisPoint = new LatLng(lat, lng);
                    LatLng aMpPoint = new LatLng(location.getLatitude(), location.getLongitude());
                    mAddress = location.getAddress();
                    if (mLocationListener != null) {
                        mLocationListener.onLocationed(aMpPoint, mAddress, arcgisPoint);
                    }
                }
            } else if (location.getErrorCode() == 12) {
                if (num == 0) {
                    ToastUtils.showToast("请打开定位权限");
                }
                num++;
            } else if (location.getErrorCode() == 4) {
                if (num1 == 0) {
                    ToastUtils.showToast("定位信号弱，未获取到位置信息！");
                }
                num1++;
            }
        }
    };

    /*地址编码，翻地址编码服务*/
    private void initAmapGeocodeSearch() {
        geocodeSearch = new GeocodeSearch(Latte.getApplicationContext());
        geocodeSearch.setOnGeocodeSearchListener(this);
    }

    public void reserveLocation(double lng, double lat) {
        LatLonPoint latLonPoint = new LatLonPoint(lat, lng);
        RegeocodeQuery query = new RegeocodeQuery(latLonPoint, 200, GeocodeSearch.GPS);
        geocodeSearch.getFromLocationAsyn(query);
    }

    public interface AmapReserveLocListener {
        void onLocationReserved(boolean isSuccess, String addr);
    }

    public void setReserveLocListener(AmapReserveLocListener mReserveLocListener) {
        this.mReserveLocListener = mReserveLocListener;
    }

    @Override
    public void onRegeocodeSearched(RegeocodeResult result, int rCode) {
        boolean isSuccess = false;
        String address = null;
        if (rCode == 1000) {
            if (result != null && result.getRegeocodeAddress() != null) {
                isSuccess = true;
                address = result.getRegeocodeAddress().getFormatAddress();
                mAddress = address;
            } else {
                ToastUtils.showToast("\"抱歉，未能找到结果\"");
            }
            if (mReserveLocListener != null) {
                mReserveLocListener.onLocationReserved(isSuccess, address);
            }
        }
    }

    @Override
    public void onGeocodeSearched(GeocodeResult geocodeResult, int i) {

    }

    public void onDestroy() {
        if (locationClient != null) {
            locationClient.stopLocation();
            locationClient.onDestroy();
        }
    }
}