package com.tofly.yxpc.map.gaode;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

import androidx.appcompat.app.AlertDialog;

import com.amap.api.maps.model.LatLng;
import com.amap.api.navi.AmapNaviType;
import com.amap.api.navi.model.NaviLatLng;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.map.LocationUtil_xu;

/**
 * @author 76486
 * 导航辅助类
 */
public class NavitHelper {

    private Context mContext = null;

    public NavitHelper(Context mContext) {
        this.mContext = mContext;
    }

    /**
     * 导航模块
     */
    public void naviSearchAddr(LatLng mLocation, LatLng toPoint) {
        if (mLocation == null) {
            ToastUtils.showToast("未获取到当前位置信息！");
            return;
        }
        if (toPoint == null) {
            ToastUtils.showToast("未获取到目的地位置信息!");
            return;
        }
        if (NetworkUtil.isNetworkAvailable()) {
            showRouteAlertDialog(mLocation, toPoint);
        } else {
            ToastUtils.showToast("暂无网络连接");
        }
    }

    /**
     * 弹窗选择 步行 驾车 导航类型
     *
     * @param mLocation 当前地址
     * @param toPoint   目的地址
     */
    private void showRouteAlertDialog(LatLng mLocation, final LatLng toPoint) {
        AlertDialog.Builder dialog = new AlertDialog.Builder(mContext).
                setTitle("请选择导航类型").
                setItems(new String[]{"步行", "驾车"},
                        (dialog1, i) -> {
                            if (i == 0) {
                                walkNaviAmap(mLocation, toPoint);
                            }
                            if (i == 1) {
                                driveNaviAmap(mLocation, toPoint);
                            }
                        });
        dialog.setCancelable(true);
        dialog.show();
    }

    /**
     * 步行导航
     */
    public void walkNaviAmap(LatLng mLocation, LatLng toPoint) {
        if (mLocation == null) {
            ToastUtils.showToast("未获取到当前位置信息!");
            return;
        }
        if (toPoint == null) {
            ToastUtils.showToast("未获取到目的地位置信息！");
            return;
        }
        LatLng ltStart = LocationUtil_xu.wgs84togcj02(mContext, mLocation.longitude, mLocation.latitude);
        LatLng ltEnd = LocationUtil_xu.wgs84togcj02(mContext, toPoint.longitude, toPoint.latitude);
        if (ltStart != null && ltEnd != null) {
            amapNaviRouteStart(AmapNaviType.WALK, ltStart, ltEnd);
        } else {
            ToastUtils.showToast("转换终点坐标失败。");
        }
    }

    /**
     * 驾车导航
     */
    public void driveNaviAmap(LatLng mLocation, LatLng toPoint) {
        if (mLocation == null) {
            ToastUtils.showToast("未获取到当前位置信息");
            return;
        }
        if (toPoint == null) {
            ToastUtils.showToast("当前任务无位置信息，无法导航!");
            return;
        }
        LatLng ltStart = LocationUtil_xu.wgs84togcj02(mContext, mLocation.longitude, mLocation.latitude);
        LatLng ltEnd = LocationUtil_xu.wgs84togcj02(mContext, toPoint.longitude, toPoint.latitude);
        if (ltStart != null && ltEnd != null) {
            amapNaviRouteStart(AmapNaviType.DRIVER, ltStart, ltEnd);
        } else {
            ToastUtils.showToast("转换终点坐标失败。");
        }
    }

    /**
     * 导航开始
     */
    public void amapNaviRouteStart(AmapNaviType naviType, LatLng ltStart, LatLng ltEnd) {
        // 起点终点坐标
        NaviLatLng mNaviStart = new NaviLatLng(ltStart.latitude, ltStart.longitude);
        NaviLatLng mNaviEnd = new NaviLatLng(ltEnd.latitude, ltEnd.longitude);

        Intent intent = new Intent();
        Bundle bundle = new Bundle();
        bundle.putParcelable(BaseAMapNaviActivity.KEY_START_LATLNG, mNaviStart);
        bundle.putParcelable(BaseAMapNaviActivity.KEY_END_LATLNG, mNaviEnd);
        intent.putExtra("isNeedShowMore", 1);
        //  intent.putExtra("adapterItem", adapterItem);
        intent.putExtras(bundle);

        if (naviType == AmapNaviType.WALK) {
            intent.setClass(mContext, WalkRouteCalculateAMapNaviActivity.class);
        } else {
            intent.setClass(mContext, SingleRouteCalculateAMapNaviActivity.class);
        }
        mContext.startActivity(intent);
    }
}
