package com.tofly.yxpc.ui.activity;

import android.widget.LinearLayout;
import android.widget.TextView;
import androidx.appcompat.widget.Toolbar;
import com.tofly.latte_core.base.LatteActivity;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.yxpc.BuildConfig;
import com.tofly.yxpc.R;
import com.tofly.yxpc.ui.impl.AboutSystemImpl;
import com.tofly.yxpc.utils.update.UpdateManager;
import com.tofly.yxpc.utils.update.VersionCallback;
import com.tofly.yxpc.utils.update.VersionInfo;
import butterknife.BindView;
import butterknife.OnClick;

/**
 * @author ychk
 */
public class AboutSystemActivity extends LatteActivity {

    @BindView(R.id.tv_toolbar_title)
    TextView tvToolbarTitle;
    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.tv_version)
    TextView tvVersion;
    @BindView(R.id.ll_version_update)
    LinearLayout llVersionUpdate;
    @BindView(R.id.tv_version_desc)
    TextView tvVersionDesc;

    private UpdateManager updateManager = null;

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new AboutSystemImpl();
    }

    @Override
    public Object setLayout() {
        return R.layout.activity_about_system;
    }

    @Override
    public void initView() {
        initToolBar(toolbar, tvToolbarTitle, "关于系统");
        initData();
    }

    private void initData() {
        updateManager = new UpdateManager(this);
        tvVersion.setText(BuildConfig.VERSION_NAME);
        updateManager.getVersionInfo(new VersionCallback() {
            @Override
            public void onGetLastVersionInfo(VersionInfo.VersionBean versionInfo) {
                String notes = "无内容";
                if (versionInfo != null) {
                    String version_desc = versionInfo.getDetail();
                    if (version_desc != null && !version_desc.isEmpty()) {
                        notes = version_desc;
                    }
                }
                tvVersionDesc.setText(notes);
            }

            @Override
            public void onError(String errMsg) {
                String notes = "无内容";
                tvVersionDesc.setText(notes);
            }
        });
    }

    @OnClick(R.id.ll_version_update)
    public void onClick() {
        if (NetworkUtil.isNetworkAvailable() && updateManager != null) {
            updateManager.checkUpdate(true);
        }
    }
}
