package com.tofly.yxpc.ui.activity;

import android.content.Intent;
import android.text.TextUtils;
import android.view.View;

import androidx.annotation.Nullable;

import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.net.bean.BaseResponse;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.yxpc.databinding.ActivityImgTextNavitBinding;
import com.tofly.yxpc.entity.FileBean;
import com.tofly.yxpc.entity.TwdhEntity;
import com.tofly.yxpc.entity.WaterMeterEntity;
import com.tofly.yxpc.greenDao.TwdhEntityDao;
import com.tofly.yxpc.ui.adapter.BigPictureAdapter;
import com.tofly.yxpc.ui.impl.UpLoadImpl;
import com.tofly.yxpc.utils.DialogWater;
import com.tofly.yxpc.utils.RecyclerViewUtils;
import com.tofly.yxpc.utils.db.DaoUtilsStore;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 76486
 */
public class ImageTextNavitActivity extends PhotoActivity {

    private ActivityImgTextNavitBinding binding;
    private BigPictureAdapter dtAdapter, dhAdapter;
    private List<FileBean> dhPhoneList, dtPhoneList;
    private Long waterMeterId;
    private long twId;
    private TwdhEntity entity;
    private String id;
    private String ids;
    private boolean isDelect = true;
    private List<WaterMeterEntity> waterMeterEntities;

    @Override
    public Object setLayout() {
        binding = ActivityImgTextNavitBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new UpLoadImpl();
    }

    @Override
    public void initView() {
        initToolBar(binding.llToolbar.toolbar, binding.llToolbar.tvToolbarTitle, "图文导航");
        waterMeterId = getIntent().getLongExtra("waterMeterId", -1);
        id = getIntent().getStringExtra("id");
        if (id != null) {
            entity = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryById(Long.parseLong(id));
        }
        if (entity == null) {
            ids = String.valueOf(waterMeterId);
            entity = new TwdhEntity();
        } else {
            ids = entity.getWaterNavigationInspects();
            binding.ll.setVisibility(View.GONE);
            binding.edName.setEnabled(false);
            binding.tvDescribe.setEnabled(false);
            isDelect = false;
            binding.tvJt.setEnabled(false);
            binding.imgPhoto.setEnabled(false);
            binding.tvWaterMeter.setEnabled(false);
        }
        initRecyclerView();
        initLister();
        loadHistory();
    }

    private void loadHistory() {
        binding.edName.setText(entity.getName());
        binding.tvDescribe.setText(entity.getDescribe());
    }

    private void initRecyclerView() {
        dhPhoneList = new ArrayList<>();
        if (entity.getFileList() != null) {
            dhPhoneList.addAll(entity.getFileList());
        }
        dhAdapter = new BigPictureAdapter(dhPhoneList, isDelect, filePathBean -> {
        });
        RecyclerViewUtils.initGrid(this, binding.rvPictureDh, 3, dhAdapter);
        if (dhPhoneList.size() == 0) {
            binding.rvPictureDh.setVisibility(View.GONE);
        } else {
            binding.rvPictureDh.setVisibility(View.VISIBLE);
        }
        dtPhoneList = new ArrayList<>();
        if (entity.getMapFileList() != null) {
            dtPhoneList.addAll(entity.getMapFileList());
        }
        dtAdapter = new BigPictureAdapter(dtPhoneList, isDelect, filePathBean -> {
        });
        RecyclerViewUtils.initGrid(this, binding.rvPictureDt, 3, dtAdapter);
        if (dtPhoneList.size() == 0) {
            binding.rvPictureDt.setVisibility(View.GONE);
        } else {
            binding.rvPictureDt.setVisibility(View.VISIBLE);
        }
        waterMeterEntities = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryAll();
        String waterMeterNums = "";
        String[] strings = ids.split(",");
        for (String s : strings) {
            for (int i = 0; i < waterMeterEntities.size(); i++) {
                if (s.equals(String.valueOf(waterMeterEntities.get(i).getId()))) {
                    if ("".equals(waterMeterNums)) {
                        waterMeterNums += waterMeterEntities.get(i).getWaterMeterNum();
                    } else {
                        waterMeterNums += "," + waterMeterEntities.get(i).getWaterMeterNum();
                    }
                    waterMeterEntities.get(i).setIscheck(true);
                }
            }
        }
        binding.tvWaterMeter.setText(waterMeterNums);
        binding.tvWaterMeter.setOnClickListener(v -> {
            DialogWater dialogWater = new DialogWater(this, (id, nums) -> {
                binding.tvWaterMeter.setText(nums);
                ids = id;
            });
            dialogWater.show();
        });
    }

    private void initLister() {
        binding.tvJt.setOnClickListener(v -> {
            if (dtPhoneList.size() < 9) {
                startActivityForResult(new Intent(ImageTextNavitActivity.this, ScreenshotActivity.class), 0);
            } else {
                onMessage("图片数量超过限制");
            }
        });
        binding.imgPhoto.setOnClickListener(v -> {
            if (dhPhoneList.size() < 9) {
                showPhoto(false);
            } else {
                onMessage("图片数量超过限制");
            }
        });
        binding.tvSave.setOnClickListener(v -> {
            if (check()) {
                save("0");
                saveBd();
                onMessage("保存成功");
            }
        });

        binding.tvCommit.setOnClickListener(v -> {
            if (check()) {
                if (NetworkUtil.isNetworkAvailable()) {
                    twId = new Date().getTime();
                    Map<String, Object> map = new HashMap<>();
                    map.put("id", twId);
                    map.put("name", binding.edName.getText().toString());
                    map.put("describe", binding.tvDescribe.getText().toString());
                    map.put("waterNavigationInspects", ids);
                    List<File> files = new ArrayList<>();
                    for (FileBean fileBean : dhPhoneList) {
                        files.add(new File(fileBean.getFilePath()));
                    }
                    map.put("file", files);
                    List<File> dtFiles = new ArrayList<>();
                    for (FileBean fileBean : dtPhoneList) {
                        dtFiles.add(new File(fileBean.getFilePath()));
                    }
                    map.put("mapfilelist", dtFiles);
                    mPresenter.getResultTwo(this, map);
                } else {
                    onMessage("网络异常，先保存本地");
                    save("0");
                    saveBd();
                    onMessage("保存成功");
                }
            }
        });
    }

    private void save(String state) {
        twId = new Date().getTime();
        entity.setId(twId);
        entity.setName(binding.edName.getText().toString());
        entity.setDescribe(binding.tvDescribe.getText().toString());
        entity.setWaterNavigationInspects(ids);
        entity.setFileList(dhPhoneList);
        entity.setMapFileList(dtPhoneList);
        entity.setState(state);
    }

    private void saveBd() {
        new Thread(() -> {
            String[] id = ids.split(",");
            for (String str : id) {
                if (!TextUtils.isEmpty(str)) {
                    List<TwdhEntity> twdhEntityList = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilder(TwdhEntityDao.Properties.WaterNavigationInspects.like("%" + str + "%"));
                    for (TwdhEntity entity1 : twdhEntityList) {
                        String waterNavigationInspects = entity1.getWaterNavigationInspects().replace("," + str, "").replace(str, "");
                        entity1.setWaterNavigationInspects(waterNavigationInspects);
                        if (TextUtils.isEmpty(entity1.getWaterNavigationInspects())) {
                            DaoUtilsStore.getInstance().getTwdhEntityDbUtils().delete(entity1);
                        } else {
                            DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertOrReplace(entity1);
                        }
                    }
                }
            }
            DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertOrReplace(entity);
            Intent intent = new Intent();
            intent.putExtra("id", twId);
            setResult(0, intent);
            finish();
        }).start();
    }

    private boolean check() {
        if (TextUtils.isEmpty(binding.edName.getText())) {
            onMessage("请输入图文导航名称");
            return false;
        }
        if (TextUtils.isEmpty(binding.edName.getText())) {
            onMessage("请输入导航描述");
            return false;
        }
        if (dtPhoneList.size() == 0) {
            onMessage("请去地图导航截取图片");
            return false;
        }
        if (TextUtils.isEmpty(ids)) {
            onMessage("请选择水表");
            return false;
        }
        return true;
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 0 && resultCode == 0) {
            if (data != null) {
                String path = data.getStringExtra("path");
                FileBean fileBean = new FileBean();
                fileBean.setFilePath(path);
                dtPhoneList.add(fileBean);
                dtAdapter.setNewData(dtPhoneList);
                if (dtPhoneList.isEmpty()) {
                    binding.rvPictureDt.setVisibility(View.GONE);
                } else {
                    binding.rvPictureDt.setVisibility(View.VISIBLE);
                }
            }
        }
    }

    @Override
    public void onTakePhotoSuccess(String pathFile) {
        FileBean fileBean = new FileBean();
        fileBean.setFilePath(pathFile);
        dhPhoneList.add(fileBean);
        dhAdapter.setNewData(dhPhoneList);
        if (dhPhoneList.size() == 0) {
            binding.rvPictureDh.setVisibility(View.GONE);
        } else {
            binding.rvPictureDh.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onTakeVideoSuccess(String videoFile) {

    }

    @Override
    public <T> void getResultTwo(T response) {
        BaseResponse baseResponse = (BaseResponse) response;
        if (baseResponse != null && "200".equals(baseResponse.getCode())) {
            onMessage("提交成功");
            save(null);
        } else {
            onMessage("提交失败，已保存到本地");
            save("0");
        }
        saveBd();
    }
}
