package com.tofly.yxpc.ui.activity;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;

import com.alibaba.fastjson.JSONArray;
import com.amap.api.maps.model.LatLng;
import com.amap.api.maps.model.Marker;
import com.amap.api.maps.model.PolygonOptions;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.databinding.ActivityLocationBinding;
import com.tofly.yxpc.map.LocationUtil_xu;
import com.tofly.yxpc.map.gaode.NavitHelper;

import java.util.ArrayList;
import java.util.List;

/**
 * @author 76486
 */
public class LocationActivity extends PhotoActivity {

    private ActivityLocationBinding binding;
    private NavitHelper navitHelper = null;
    private String isRed;
    /**
     * 任务点跳转值
     */
    private LocationUtil_xu.CoordinateClass loTask = null;
    private Marker locationMarker = null;
    private Marker locationMarker1 = null;
    /**
     * 阳新定位点
     */
    private LatLng taskPoint = null;
    private boolean isDrag = true;

    @Override
    public void onTakePhotoSuccess(String pathFile) {

    }

    @Override
    public void onTakeVideoSuccess(String videoFile) {

    }

    @Override
    public Object setLayout() {
        binding = ActivityLocationBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return null;
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        setIsLocation(false);
        super.onCreate(savedInstanceState);
    }

    @Override
    public void initView() {
        loTask = (LocationUtil_xu.CoordinateClass) getIntent().getSerializableExtra("locationUtilXuNew");
        isRed = getIntent().getStringExtra("isRed");
        isDrag = getIntent().getBooleanExtra("isDrag", true);
        navitHelper = new NavitHelper(this);
        initToolBar(binding.llToolbar.toolbar, binding.llToolbar.tvToolbarTitle, isDrag ? "水表定位" : "任务位置");
        switch (isRed) {
            case "0":
            case "1":
                break;
            case "2":
                setIsLocation(true);
                break;
        }
        setType(isRed);
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    public void initMapView() {
        super.initMapView();
        aMap.setOnMapTouchListener(motionEvent -> binding.mapView.requestDisallowInterceptTouchEvent(motionEvent.getAction() != MotionEvent.ACTION_UP));
        if (isDrag) {
            aMap.setOnCameraChangeListener(mapCameraChangeListener);
            binding.imDrag.setVisibility(View.VISIBLE);
            binding.imgNavit.setVisibility(View.GONE);
            binding.imgFw.setVisibility(View.GONE);
            binding.tvLat.setVisibility(View.VISIBLE);
            binding.tvSure.setVisibility(View.VISIBLE);
        } else {
            binding.imDrag.setVisibility(View.GONE);
            binding.imgFw.setVisibility(View.VISIBLE);
            binding.imgNavit.setVisibility(View.VISIBLE);
            binding.tvLat.setVisibility(View.GONE);
            binding.tvSure.setVisibility(View.GONE);
        }

        binding.imgLocation.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime() && lo != null) {
                moveOrigin(lo.Longtitude, lo.Latitude, aMap.getCameraPosition().zoom);
                if (locationMarker != null) {
                    locationMarker.remove();
                }
                if (isDrag) {
                    locationMarker1.remove();
                }
                locationMarker = aMap.addMarker(setMark(lo.Longtitude, lo.Latitude, "1"));
            }
        });
        binding.imgMap.setOnClickListener(v -> {
            checkMap();
        });
        binding.imgFw.setOnClickListener(v -> {
            moveOrigin(taskPoint.longitude, taskPoint.latitude, aMap.getCameraPosition().zoom);
        });
        binding.tvSure.setOnClickListener(v -> {
            Intent intent = new Intent();
            intent.putExtra("locationUtilXuNew", loTask);
            setResult(1106, intent);
            finish();
        });
        binding.imgNavit.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime() && lo != null && loTask != null) {
                navitHelper.naviSearchAddr(new LatLng(lo.Latitude, lo.Longtitude), new LatLng(loTask.Latitude, loTask.Longtitude));
            } else {
                onMessage("当前任务无位置信息，无法导航！");
            }
        });
        if (loTask != null) {
            taskPoint = new LatLng(loTask.Latitude, loTask.Longtitude);
        }
        String str = getIntent().getStringExtra("latLngs");
        if (str != null && str.length() > 2) {
            List<LatLng> latLngs = new ArrayList<>();
            List<ArrayList> list = JSONArray.parseArray(str, ArrayList.class);
            for (ArrayList bean : list) {
                latLngs.clear();
                List<String[]> strings = JSONArray.parseArray(bean.toString(), String[].class);
                for (String[] strings1 : strings) {
                    double longitude = Double.parseDouble(strings1[0]);
                    double latitude = Double.parseDouble(strings1[1]);
                    latLngs.add(new LatLng(latitude, longitude));
                }
                // 声明 多边形参数对象
                PolygonOptions polygonOptions = new PolygonOptions();
// 添加 多边形的每个顶点（顺序添加）
                polygonOptions.addAll(latLngs);
                polygonOptions.strokeWidth(5) // 多边形的边框
                        .strokeColor(getColor(R.color.red2)) // 边框颜色
                        .fillColor(getColor(R.color.red3));   // 多边形的填充色
                aMap.addPolygon(polygonOptions);
            }
            if (loTask == null) {
                if (!latLngs.isEmpty()) {
                    LatLng center = latLngs.get(0);
                    if (center != null) {
                        taskPoint = new LatLng(center.latitude, center.longitude);
                        moveOrigin(taskPoint.longitude, taskPoint.latitude, 16);
                    }
                } else {
                    taskPoint = new LatLng(29.83062932, 115.21583523);
                    moveOrigin(taskPoint.longitude, taskPoint.latitude, 20);
                }
            } else {
                moveOrigin(taskPoint.longitude, taskPoint.latitude, 16);
                locationMarker1 = aMap.addMarker(setMark(taskPoint.longitude, taskPoint.latitude, isRed));
            }
        } else {
            if (loTask == null) {
                taskPoint = new LatLng(29.83062932, 115.21583523);
                moveOrigin(taskPoint.longitude, taskPoint.latitude, 20);
            } else {
                moveOrigin(taskPoint.longitude, taskPoint.latitude, 20);
                locationMarker1 = aMap.addMarker(setMark(taskPoint.longitude, taskPoint.latitude, isRed));
            }
        }
        if (loTask != null) {
            binding.tvLat.setText("水表位置         " + "经度: " + loTask.Longtitude + " 纬度: " + loTask.Latitude);
        }
    }

    @Override
    public void initMapListener(LocationUtil_xu.CoordinateClass info) {
        super.initMapListener(info);
        loTask = info;
        binding.tvLat.setText("水表位置         " + "经度: " + loTask.Longtitude + " 纬度: " + loTask.Latitude);
    }

}
