package com.tofly.yxpc.ui.activity;

import android.Manifest;
import android.app.AlertDialog;
import android.content.Intent;
import android.text.TextUtils;
import android.text.method.PasswordTransformationMethod;
import android.text.method.TransformationMethod;

import com.tofly.latte_core.base.LatteActivity;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.databinding.ActivityLoginBinding;
import com.tofly.yxpc.entity.user.LoginBean;
import com.tofly.yxpc.entity.user.UserInfo;
import com.tofly.yxpc.ui.impl.LoginImpl;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.yxpc.widget.PasswordDialog;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author ychk
 */
public class LoginActivity extends LatteActivity {

    private ActivityLoginBinding binding;
    private String pws = "";

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new LoginImpl();
    }

    @Override
    public Object setLayout() {
        binding = ActivityLoginBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    public void initView() {
        checkPermission(isSuccess -> {
                }, Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.CAMERA,
                Manifest.permission.CALL_PHONE,
                Manifest.permission.RECORD_AUDIO,
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.READ_EXTERNAL_STORAGE);
        if (AccountManager.getIsSave()) {
            binding.etAccount.setText(AccountManager.getUsername() == null ? "" : AccountManager.getUsername());
            binding.etPsw.setText(AccountManager.getUserPassword() == null ? "" : AccountManager.getUserPassword());
        } else {
            binding.etAccount.setText("");
            binding.etPsw.setText("");
        }
        if (getIntent().getBooleanExtra("isLoss", false)) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this, R.style.AlertDialog);
            builder.setTitle("授权失败")
                    .setMessage("身份验证失败，请重新登录")
                    .setCancelable(false)
                    .setPositiveButton("确定", null)
                    .create()
                    .show();
        }
        initListener();
    }

    private void initListener() {
        binding.tvSubmit.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime() && checkData()) {
                login();
            }
        });
        binding.imgEye.setOnClickListener(v -> {
            TransformationMethod transformationMethod = binding.etPsw.getTransformationMethod();
            if (transformationMethod instanceof PasswordTransformationMethod) {
                binding.etPsw.setTransformationMethod(null);
            } else {
                binding.etPsw.setTransformationMethod(PasswordTransformationMethod.getInstance());
            }
        });
    }

    private boolean checkData() {
        boolean isSuccess = false;
        if (TextUtils.isEmpty(binding.etAccount.getText())) {
            binding.etAccount.requestFocus();
            binding.etAccount.setError("账号为空");
        } else if (TextUtils.isEmpty(binding.etPsw.getText())) {
            binding.etPsw.requestFocus();
            binding.etPsw.setError("密码为空");
        } else {
            isSuccess = true;
        }
        return isSuccess;
    }

    /**
     * 修改密码
     */
    private void showMyUpDatePasswordDialog(UserInfo bean) {
        PasswordDialog passwordDialog = new PasswordDialog(this, false, (oldPass, newPass) -> {
            Map<String, Object> map = new HashMap<>();
            map.put("userId", bean.getUserId() == null ? "" : bean.getUserId());
            map.put("userName", bean.getUsername() == null ? "" : bean.getUsername());
            map.put("password", oldPass);
            map.put("resetPassword", newPass);
            mPresenter.getResultThree(this, map);
            pws = newPass;
            AccountManager.setUsername(bean.getUsername());
        });
        passwordDialog.show();
    }

    /**
     * 保存个人信息
     */
    private void saveUser(UserInfo bean) {
        AccountManager.setUsername(bean.getUsername());
        AccountManager.setUserPassword(binding.etPsw.getText().toString());
        AccountManager.setUserId(bean.getUserId());
        AccountManager.setAvatar(bean.getAvatar());
        AccountManager.setPhone(bean.getPhoneNumber());
        AccountManager.setUserRealName(bean.getRealName());

        intoMainActivity();
    }

    private void login() {
        if (!NetworkUtil.isNetworkAvailable()) {
            if (AccountManager.getUsername().equals(binding.etAccount.getText().toString()) && AccountManager.getUserPassword().equals(binding.etPsw.getText().toString())) {
                intoMainActivity();
            } else {
                onMessage("账号或密码错误");
            }
        } else {
            Map<String, Object> map = new HashMap<>(5);
            map.put("username", binding.etAccount.getText().toString());
            map.put("password", binding.etPsw.getText().toString());
            mPresenter.getResultOne(this, map);
        }
    }

    /**
     * 跳转到主界面
     */
    private void intoMainActivity() {
        Intent intent = new Intent(this, MainActivity.class);
        startActivity(intent);
        finish();
    }

    @Override
    public <T> void getResultOne(T response) {
        if (response != null) {
            LoginBean loginBean = (LoginBean) response;
            if (loginBean.getToken() != null) {
                AccountManager.setToken(loginBean.getToken());
                Map<String, Object> map = new HashMap<>(2);
                map.put("time", new Date().getTime());
                mPresenter.getResultTwo(this, map);
            }
        }
    }

    @Override
    public <T> void getResultTwo(T response) {
        if (response != null) {
            UserInfo userEntity = (UserInfo) response;
            if (userEntity.getNeedSetPwd() != null && userEntity.getNeedSetPwd()) {
                showMyUpDatePasswordDialog(userEntity);
            } else {
                saveUser(userEntity);
            }
        }
    }

    @Override
    public <T> void getResultThree(T response) {
        super.getResultThree(response);
        if (response != null) {
            AccountManager.setUserPassword(pws);
            binding.etPsw.setText(pws);
            login();
        } else {
            AccountManager.setUserPassword("");
        }
    }

}
