package com.tofly.yxpc.ui.activity;

import android.text.TextUtils;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;

import com.tofly.latte_core.base.LatteActivity;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.latte_core.utils.ActivityUtils;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.databinding.ActivityMainBinding;
import com.tofly.yxpc.entity.DictEntity;
import com.tofly.yxpc.entity.TwdhEntity;
import com.tofly.yxpc.entity.WaterMeterEntity;
import com.tofly.yxpc.entity.WaterMeterTaskEntity;
import com.tofly.yxpc.greenDao.DictEntityDao;
import com.tofly.yxpc.greenDao.TwdhEntityDao;
import com.tofly.yxpc.greenDao.WaterMeterTaskEntityDao;
import com.tofly.yxpc.ui.fragment.MeFragment;
import com.tofly.yxpc.ui.fragment.DataUpdateFragment;
import com.tofly.yxpc.ui.fragment.IndexFragment;
import com.tofly.yxpc.ui.impl.MainImpl;
import com.tofly.yxpc.utils.db.DaoUtilsStore;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ychk
 */
public class MainActivity extends LatteActivity {

    private ActivityMainBinding binding;
    private Fragment mFragment = null;
    private IndexFragment indexFragment = null;
    private DataUpdateFragment dataUpdateFragment = null;
    private MeFragment meFragment = null;

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new MainImpl();
    }

    @Override
    public Object setLayout() {
        binding = ActivityMainBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    public void initView() {
        indexFragment = new IndexFragment(() -> {
            binding.rbtDataUpdate.setChecked(true);
            setFragment(dataUpdateFragment);
        });
        dataUpdateFragment = new DataUpdateFragment();
        meFragment = new MeFragment();
        setFragment(indexFragment);
        initListener();
        if (NetworkUtil.isNetworkAvailable()) {
            showLoading("任务下载中...");
            Map<String, Object> map = new HashMap<>();
            map.put("userId", AccountManager.getUserId());
            mPresenter.getResultThree(this, map);
        }
    }

    @Override
    protected void onRestart() {
        super.onRestart();
    }

    private void initListener() {
        binding.rbtIndex.setOnClickListener(v -> {
            indexFragment.initView();
            setFragment(indexFragment);
        });
        binding.rbtDataUpdate.setOnClickListener(v -> setFragment(dataUpdateFragment));
        binding.rbtMe.setOnClickListener(v -> setFragment(meFragment));
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (NetworkUtil.isNetworkAvailable()) {
            Map<String, Object> m = new HashMap<>(4);
            m.put("dictCode", "water_nature");
            mPresenter.getResultOne(this, m);

            m.clear();
            m.put("dictCode", "abnormal_info");
            mPresenter.getResultOne(this, m);

            m.clear();
            m.put("dictCode", "meter_brand");
            mPresenter.getResultOne(this, m);

            m.clear();
            m.put("dictCode", "registered_type");
            mPresenter.getResultOne(this, m);

            m.clear();
            m.put("dictCode", "noregistered_type");
            mPresenter.getResultOne(this, m);

            m.clear();
            m.put("dictCode", "enroll_info");
            mPresenter.getResultOne(this, m);

            m.clear();
            m.put("dictCode", "water_nature");
            mPresenter.getResultOne(this, m);

            Map<String, Object> m1 = new HashMap<>(4);
            m1.put("checkUser", AccountManager.getUserId());
            mPresenter.getResultTwo(this, m1);
        }
    }

    /**
     * 添加fragment显示
     */
    private void setFragment(Fragment fragment) {
        FragmentTransaction transaction = this.getSupportFragmentManager().beginTransaction();
        if (mFragment == null) {
            transaction.add(R.id.fl_content, fragment).commit();
        } else {
            if (mFragment != fragment) {
                if (fragment.isAdded()) {
                    transaction.hide(mFragment).show(fragment).commit();
                } else {
                    transaction.hide(mFragment).add(R.id.fl_content, fragment).commit();
                }
            }
        }
        mFragment = fragment;
    }

    @Override
    public void onBackPressed() {
        if (!MyApp.getInstance().isOverTime()) {
            ActivityUtils.getInstance().finishAllActivity();
        } else {
            onMessage("再按一次退出程序");
        }
    }

    @Override
    public <T> void getResultOne(T response) {
        if (response != null) {
            List<DictEntity> dictBeans = (List<DictEntity>) response;
            if (!dictBeans.isEmpty()) {
                for (DictEntity entity : dictBeans) {
                    entity.setDictCode(entity.getSysDict().getDictCode());
                    entity.setDictName(entity.getSysDict().getDictName());
                }
                List<DictEntity> dictList = DaoUtilsStore.getInstance().getDictEntityDbUtils().queryByQueryBuilder(DictEntityDao.Properties.DictCode.eq(dictBeans.get(0).getDictCode()));
                DaoUtilsStore.getInstance().getDictEntityDbUtils().delete(dictList);
            }
            DaoUtilsStore.getInstance().getDictEntityDbUtils().insertMulti(dictBeans, false);
        }
    }

    @Override
    public <T> void getResultTwo(T response) {
        if (response != null) {
            List<TwdhEntity> twdhEntities = (List<TwdhEntity>) response;
            if (!twdhEntities.isEmpty()) {
                List<TwdhEntity> twdhEntityList = new ArrayList<>(twdhEntities);
                for (int i = 0; i < twdhEntityList.size(); i++) {
                    String[] ids = twdhEntityList.get(i).getWaterNavigationInspects().split(",");
                    for (String s : ids) {
                        if (!TextUtils.isEmpty(s)) {
                            TwdhEntity twdhEntity = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilderOne(TwdhEntityDao.Properties.WaterNavigationInspects.like("%" + s + "%"));
                            if (twdhEntity != null) {
                                twdhEntities.remove(twdhEntityList.get(i));
                                break;
                            }
                        }
                    }
                }
                DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertMulti(twdhEntities, false);
            }
        }
    }

    @Override
    public <T> void getResultThree(T response) {
        hideLoading();
        if (response != null) {
            DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().delete(DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().queryByQueryBuilder(WaterMeterTaskEntityDao.Properties.CheckUser.eq(AccountManager.getUserId())));
            List<WaterMeterTaskEntity> waterMeterTaskEntities = (List<WaterMeterTaskEntity>) response;
            DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().insertMulti(waterMeterTaskEntities, true);
            for (WaterMeterTaskEntity waterMeterTaskEntity : waterMeterTaskEntities) {
                if (waterMeterTaskEntity.getYxswWaterMeterInspectDto() != null && !waterMeterTaskEntity.getYxswWaterMeterInspectDto().isEmpty()) {
                    for (WaterMeterEntity sbdjEntity : waterMeterTaskEntity.getYxswWaterMeterInspectDto()) {
                        sbdjEntity.setTaskId(waterMeterTaskEntity.getId());
                        sbdjEntity.setCheckUser(waterMeterTaskEntity.getCheckUser());
                        WaterMeterEntity dbWaterMeter = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryById(sbdjEntity.getId());
                        if (dbWaterMeter == null || dbWaterMeter.getState() == 0 || dbWaterMeter.getState() == 3) {
                            DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().insertOrReplace(sbdjEntity);
                        }
                    }
                }
            }
            onMessage("任务下载成功");
            indexFragment.initView();
        }
    }
}