package com.tofly.yxpc.ui.activity;

import android.content.Intent;
import android.text.TextUtils;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.RequestOptions;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.latte_core.utils.ActivityUtils;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.NetWorkUrl;
import com.tofly.yxpc.databinding.ActivitySettingBinding;
import com.tofly.yxpc.entity.base.FileResponse;
import com.tofly.yxpc.ui.impl.SettingImpl;
import com.tofly.yxpc.utils.CheckPhoneUtils;
import com.tofly.yxpc.utils.ImageUtil;
import com.tofly.yxpc.utils.RequestBodyUtils;
import com.tofly.yxpc.widget.PasswordDialog;
import com.tofly.yxpc.widget.UserInfoDialog;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * @author 76486
 */
public class SettingActivity extends PhotoActivity {

    private ActivitySettingBinding binding;
    private String pwd;
    private ImageUtil imageUtil = null;
    private String img = null;

    @Override
    public Object setLayout() {
        binding = ActivitySettingBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new SettingImpl();
    }

    @Override
    public void onTakePhotoSuccess(String pathFile) {
        File file = new File(pathFile);
        if (file.exists()) {
            Map<String, Object> map = new HashMap<>();
            map.put("file\";filename=\"" + file.getName(), RequestBodyUtils.toRequestBodyFile(file, this));
            mPresenter.getResultOne(this, map);
        }
    }

    @Override
    public void onTakeVideoSuccess(String videoFile) {

    }

    @Override
    public void initView() {
        initToolBar(binding.llToolbar.toolbar, binding.llToolbar.tvToolbarTitle, "设置");
        imageUtil = new ImageUtil(this);
        if (!TextUtils.isEmpty(NetWorkUrl.getFileUrl(AccountManager.getAvatar()))) {
            Glide.with(this)
                    .load(NetWorkUrl.getFileUrl(AccountManager.getAvatar()))
                    .apply(new RequestOptions().error(R.mipmap.my_heard))
                    .into(binding.circleImgPortrait);
        }
        binding.tvName.setText(AccountManager.getUserRealName() == null ? "" : AccountManager.getUserRealName());
        binding.tvLxdh.setText(AccountManager.getPhone() == null ? "" : AccountManager.getPhone());
        initListener();
    }

    private void initListener() {
        binding.circleImgPortrait.setOnClickListener(v -> {
            if (!TextUtils.isEmpty(AccountManager.getAvatar())) {
                imageUtil.showBigImage(NetWorkUrl.getFileUrl(AccountManager.getAvatar()), false);
            }
        });
        binding.imgPortrait.setOnClickListener(v -> {
            showPhoto(true, false);
        });
        binding.imgName.setOnClickListener(v -> {
            showChangeUserInfoDialog(UserInfoDialog.INPUT_NAME);
        });
        binding.imgLxdh.setOnClickListener(v -> {
            showChangeUserInfoDialog(UserInfoDialog.INPUT_MOBILE);
        });
        binding.imgPwd.setOnClickListener(v -> {
            PasswordDialog passwordDialog = new PasswordDialog(this, false, (oldPass, newPass) -> {
                Map<String, Object> map = new HashMap<>();
                map.put("userId", AccountManager.getUserId() == null ? "" : AccountManager.getUserId());
                map.put("userName", AccountManager.getUsername() == null ? "" : AccountManager.getUsername());
                map.put("password", oldPass);
                map.put("resetPassword", newPass);
                mPresenter.getResultThree(this, map);
                pwd = newPass;
            });
            passwordDialog.show();
        });
    }

    private void showChangeUserInfoDialog(int inputType) {
        String userName = binding.tvName.getText().toString();
        String userPhone = binding.tvLxdh.getText().toString();
        UserInfoDialog userInfoDialog = new UserInfoDialog(this, userName, userPhone, "", inputType);
        userInfoDialog.setClickListener(new UserInfoDialog.ClickListener() {

            @Override
            public void doOk(String name, String mobile, String email) {
                Map<String, Object> map = new HashMap<>();
                if (inputType == UserInfoDialog.INPUT_NAME) {
                    binding.tvName.setText(name);
                    map.put("realName", name);
                    map.put("phoneNumber", AccountManager.getPhone());
                    if (!TextUtils.isEmpty(AccountManager.getAvatar())) {
                        map.put("avatar", AccountManager.getAvatar());
                    }
                    loadData(map);
                } else if (inputType == UserInfoDialog.INPUT_MOBILE) {
                    if (!TextUtils.isEmpty(mobile)) {
                        if (CheckPhoneUtils.isPhoneNum(mobile)) {
                            binding.tvLxdh.setText(mobile);
                            map.put("phoneNumber", mobile);
                            map.put("realName", AccountManager.getUserRealName());
                            if (!TextUtils.isEmpty(AccountManager.getAvatar())) {
                                map.put("avatar", AccountManager.getAvatar());
                            }
                            loadData(map);
                        } else {
                            onMessage("请输入正确的电话号码");
                        }
                    }
                }
            }

            @Override
            public void doCancel() {
            }
        });
        userInfoDialog.show();
    }

    private void loadData(Map<String, Object> map) {
        map.put("id", AccountManager.getUserId());
        mPresenter.getResultTwo(this, map);
    }

    @Override
    public <T> void getResultOne(T response) {
        super.getResultOne(response);
        if (response != null) {
            FileResponse fileResponse = (FileResponse) response;
            if (!TextUtils.isEmpty(fileResponse.getFileName())) {
                Glide.with(this)
                        .load(NetWorkUrl.getFileUrl(fileResponse.getFileName()))
                        .apply(new RequestOptions().error(R.mipmap.my_heard))
                        .into(binding.circleImgPortrait);
                Map<String, Object> map = new HashMap<>();
                map.put("realName", AccountManager.getUserRealName());
                map.put("phoneNumber", AccountManager.getPhone());
                map.put("avatar", fileResponse.getFileName());
                img = "/" + fileResponse.getFileName();
                loadData(map);
            }
        }
    }

    @Override
    public <T> void getResultTwo(T response) {
        super.getResultTwo(response);
        if (response != null) {
            onMessage("更新成功");
            AccountManager.setPhone(binding.tvLxdh.getText().toString());
            AccountManager.setUserRealName(binding.tvName.getText().toString());
            if (!TextUtils.isEmpty(img)) {
                AccountManager.setAvatar(img);
            }
        } else {
            binding.tvLxdh.setText(AccountManager.getPhone());
            binding.tvName.setText(AccountManager.getUserRealName());
        }
    }

    @Override
    public <T> void getResultThree(T response) {
        super.getResultThree(response);
        if (response != null) {
            onMessage("更新成功");
            AccountManager.setUserPassword(pwd);
            startActivity(new Intent(this, LoginActivity.class));
            ActivityUtils.getInstance().finishAllActivity();
        }
    }
}
