package com.tofly.yxpc.ui.adapter;

import android.app.Activity;
import android.content.Intent;
import android.text.TextUtils;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.bumptech.glide.Glide;
import com.chad.library.adapter.base.BaseQuickAdapter;
import com.chad.library.adapter.base.BaseViewHolder;
import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.NetWorkUrl;
import com.tofly.yxpc.entity.FileBean;
import com.tofly.yxpc.ui.activity.VideoActivity;
import com.tofly.yxpc.utils.DialogUtils;
import com.tofly.yxpc.utils.ImageUtil;
import com.tofly.yxpc.utils.MP3Player;
import com.tofly.yxpc.utils.WpsUtils;

import java.io.File;
import java.util.List;

/**
 * @author ychk
 */
public class PictureAdapter extends BaseQuickAdapter<FileBean, BaseViewHolder> implements WpsUtils.WpsInterface {

    private final boolean isDelete;
    private WpsUtils wpsUtil;
    private MP3Player mp3Player;

    public PictureAdapter(@Nullable List<FileBean> data, boolean isDelete) {
        super(R.layout.item_picture, data);
        this.isDelete = isDelete;
    }

    @Override
    protected void convert(BaseViewHolder helper, FileBean item) {
        if (!TextUtils.isEmpty(item.getFilePath())) {
            File file = new File(item.getFilePath());
            String[] arr = file.getName().split("\\.");
            String type = arr[arr.length - 1];
            ImageView imageView = helper.getView(R.id.iv_picture);
            TextView tvFile = helper.getView(R.id.tv_file);
            switch (type) {
                case "jpg":
                case "png":
                case "jpeg":
                case "webp":
                case "gif":
                    tvFile.setVisibility(View.GONE);
                    imageView.setVisibility(View.VISIBLE);
                    if (!file.exists()) {
                        Glide.with(mContext)
                                .load(NetWorkUrl.getFileUrl(item.getFilePath()))
                                .into((imageView));
                        imageView.setOnClickListener(v -> {
                            ImageUtil imageUtil = new ImageUtil(mContext);
                            imageUtil.showBigImage(NetWorkUrl.getFileUrl(item.getFilePath()), false);
                        });
                    } else {
                        Glide.with(mContext)
                                .load(item.getFilePath())
                                .into((imageView));
                        imageView.setOnClickListener(v -> {
                            ImageUtil imageUtil = new ImageUtil(mContext);
                            imageUtil.showBigImage(item.getFilePath(), false);
                        });
                    }
                    break;
                case "doc":
                case "docx":
                case "xls":
                case "xlsx":
                case "pdf":
                case "txt":
                case "ppt":
                case "pptx":
                    tvFile.setText(item.getFileName() == null ? "" : item.getFileName());
                    tvFile.setVisibility(View.VISIBLE);
                    imageView.setVisibility(View.GONE);
                    tvFile.setOnClickListener(v -> wpsUtil = new WpsUtils(this, file, false, (Activity) mContext));
                    break;
                case "amr":
                    //这里是处理录音类文件
                    tvFile.setVisibility(View.GONE);
                    imageView.setVisibility(View.VISIBLE);
                    //加载默认音频文件图标
                    imageView.setImageResource(R.mipmap.record_file);
                    imageView.setOnClickListener(v -> {
                        //点击音频实现播放
                        if (mp3Player == null) {
                            mp3Player = new MP3Player(mContext);
                        }
                        String pathUrl = item.getFilePath();
                        if (!file.exists()) {
                            pathUrl = NetWorkUrl.getFileUrl(file.getPath());
                        }
                        if (pathUrl.startsWith("http")) {
                            mp3Player.initInternetUrl(pathUrl);
                        } else {
                            mp3Player.init(pathUrl);
                        }
                        mp3Player.play();

                    });
                    break;
                case "mp4":
                    tvFile.setVisibility(View.GONE);
                    imageView.setVisibility(View.VISIBLE);
                    Glide.with(mContext)
                            .load(item.getFilePath())
                            .into(imageView);
                    helper.setOnClickListener(R.id.iv_picture, v -> {
                        Intent intent = new Intent(mContext, VideoActivity.class);
                        intent.putExtra("videoPath", item.getFilePath());
                        mContext.startActivity(intent);
                    });
                    break;
                default:
                    tvFile.setVisibility(View.VISIBLE);
                    tvFile.setText("无法识别的文件格式");
                    imageView.setVisibility(View.GONE);
                    helper.setOnClickListener(R.id.tv_file, v -> ToastUtils.showToast("无法识别该格式文件"));
                    break;
            }
        }
        ImageView delete = helper.getView(R.id.img_delete);
        if (isDelete) {
            delete.setVisibility(View.VISIBLE);
        } else {
            delete.setVisibility(View.GONE);
        }
        delete.setOnClickListener(v -> {
            DialogUtils.showDialog(mContext, "是否确认删除", "", true, (dialog, which) -> {
                mData.remove(item);
                notifyDataSetChanged();
            });
        });
    }

    @Override
    public void doRequest(String filePath) {

    }

    @Override
    public void doFinish() {
        wpsUtil.appBack();
    }

}
