package com.tofly.yxpc.ui.fragment;

import android.content.Intent;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;

import androidx.annotation.NonNull;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.base.LatteDelegate;
import com.tofly.latte_core.net.bean.BaseResponse;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.databinding.FragmentDataUpdateBinding;
import com.tofly.yxpc.entity.BaseUpdataEntity;
import com.tofly.yxpc.entity.FileBean;
import com.tofly.yxpc.entity.SbdjEntity;
import com.tofly.yxpc.entity.TwdhEntity;
import com.tofly.yxpc.entity.WaterMeterEntity;
import com.tofly.yxpc.entity.WaterMeterTaskEntity;
import com.tofly.yxpc.greenDao.BaseUpdataEntityDao;
import com.tofly.yxpc.greenDao.SbdjEntityDao;
import com.tofly.yxpc.greenDao.TwdhEntityDao;
import com.tofly.yxpc.greenDao.WaterMeterEntityDao;
import com.tofly.yxpc.greenDao.WaterMeterTaskEntityDao;
import com.tofly.yxpc.ui.activity.TaskDetailActivity;
import com.tofly.yxpc.ui.activity.UnRegisterActivity;
import com.tofly.yxpc.ui.adapter.DataUpdateAdapter;
import com.tofly.yxpc.ui.impl.UpLoadImpl;
import com.tofly.yxpc.utils.DialogUtils;
import com.tofly.yxpc.utils.RecyclerViewUtils;
import com.tofly.yxpc.utils.SpinnerUtils;
import com.tofly.yxpc.utils.db.DaoUtilsStore;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ychk
 */
public class DataUpdateFragment extends LatteDelegate {

    private FragmentDataUpdateBinding binding;
    private DataUpdateAdapter adapter;
    private final List<BaseUpdataEntity> list = new ArrayList<>();
    private int type = -1;
    private boolean isFirstLoading = true;
    private List<SbdjEntity> sbdjEntities;
    private List<TwdhEntity> twdhEntities;
    private List<WaterMeterEntity> waterMeterEntities;
    private int page = 0;

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new UpLoadImpl();
    }

    @Override
    public Object setLayout() {
        binding = FragmentDataUpdateBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    public void initView() {
        initToolBar(binding.llToolbar.toolbar, binding.llToolbar.tvToolbarTitle, "数据上传");
        initRecyclerView();
        initList();
        initListener();
    }

    private void initListener() {
        binding.tvAllUpdate.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime()) {
                if (NetworkUtil.isNetworkAvailable()) {
                    sbdjEntities = DaoUtilsStore.getInstance().getSbdjEntityDbUtils().queryByQueryBuilder(SbdjEntityDao.Properties.CheckId.eq(AccountManager.getUserId()));
                    waterMeterEntities = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryByQueryBuilder(WaterMeterEntityDao.Properties.State.eq(2), WaterMeterEntityDao.Properties.CheckUser.eq(AccountManager.getUserId()));
                    twdhEntities = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilder(TwdhEntityDao.Properties.State.eq("0"));
                    if ((sbdjEntities == null || sbdjEntities.isEmpty()) && (twdhEntities == null || twdhEntities.isEmpty()) && (waterMeterEntities == null || waterMeterEntities.isEmpty())) {
                        onMessage("暂无数据上传...");
                    } else {
                        DialogUtils.showDialog(requireContext(), "提示", "是否上传数据", false, (dialog, which) -> {
                            if (sbdjEntities != null && sbdjEntities.size() > 0) {
                                showLoading("数据上传中...");
                                page = 0;
                                updataSbdj();
                            } else if (twdhEntities != null && !twdhEntities.isEmpty()) {
                                showLoading("数据上传中...");
                                page = 0;
                                updataTwdh();
                            } else {
                                if (waterMeterEntities != null && !waterMeterEntities.isEmpty()) {
                                    showLoading("数据上传中...");
                                    page = 0;
                                    updataWaterMeter();
                                }
                            }
                        });
                    }
                } else {
                    onMessage("网络异常");
                }
            }
        });
    }

    @Override
    public void onResume() {
        super.onResume();
        if (!isFirstLoading) {
            //如果不是第一次加载，刷新数据
            initList();
        }
        isFirstLoading = false;
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        // inflater.inflate(R.menu.menu_all_update, menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        if (NetworkUtil.isNetworkAvailable()) {
            sbdjEntities = DaoUtilsStore.getInstance().getSbdjEntityDbUtils().queryByQueryBuilder(SbdjEntityDao.Properties.CheckId.eq(AccountManager.getUserId()));
            waterMeterEntities = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryByQueryBuilder(WaterMeterEntityDao.Properties.State.eq(2), WaterMeterEntityDao.Properties.CheckUser.eq(AccountManager.getUserId()));
            twdhEntities = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilder(TwdhEntityDao.Properties.State.eq("0"));
            if ((sbdjEntities == null || sbdjEntities.size() == 0) && (twdhEntities == null || twdhEntities.size() == 0) && (waterMeterEntities == null || waterMeterEntities.size() == 0)) {
                onMessage("暂无数据上传...");
            } else {
                DialogUtils.showDialog(requireContext(), "提示", "是否上传数据", false, (dialog, which) -> {
                    if (sbdjEntities != null && sbdjEntities.size() > 0) {
                        showLoading("数据上传中...");
                        page = 0;
                        updataSbdj();
                    } else if (twdhEntities != null && twdhEntities.size() > 0) {
                        showLoading("数据上传中...");
                        page = 0;
                        updataTwdh();
                    } else {
                        if (waterMeterEntities != null && waterMeterEntities.size() > 0) {
                            showLoading("数据上传中...");
                            page = 0;
                            updataWaterMeter();
                        }
                    }
                });
            }
        } else {
            onMessage("网络异常");
        }
        return super.onOptionsItemSelected(item);
    }

    public void updataSbdj() {
        Map map = JSONObject.parseObject(JSON.toJSONString(sbdjEntities.get(page)), HashMap.class);
        List<File> files = new ArrayList<>();
        for (FileBean fileBean : sbdjEntities.get(page).getFile()) {
            files.add(new File(fileBean.getFilePath()));
        }
        map.put("file", files);
        List<File> files1 = new ArrayList<>();
        if (sbdjEntities.get(page).getBpFileList() != null) {
            for (FileBean fileBean : sbdjEntities.get(page).getBpFileList()) {
                files1.add(new File(fileBean.getFilePath()));
            }
            map.put("bpFileList", files1);
        }
        mPresenter.getResultOne(requireContext(), map);
    }

    public void updataTwdh() {
        Map map = new HashMap();
        map.put("name", twdhEntities.get(page).getName());
        map.put("describe", twdhEntities.get(page).getDescribe());
        map.put("waterNavigationInspects", twdhEntities.get(page).getWaterNavigationInspects());
        List<File> files = new ArrayList<>();
        for (FileBean fileBean : twdhEntities.get(page).getFileList()) {
            files.add(new File(fileBean.getFilePath()));
        }
        map.put("file", files);
        files.clear();
        for (FileBean fileBean : twdhEntities.get(page).getMapFileList()) {
            files.add(new File(fileBean.getFilePath()));
        }
        map.put("mapfilelist", files);
        mPresenter.getResultTwo(getContext(), map);
    }

    private void updataWaterMeter() {
        Map map = JSONObject.parseObject(JSON.toJSONString(waterMeterEntities.get(page)), HashMap.class);
        List<File> files1 = new ArrayList<>();
        if (waterMeterEntities.get(page).getBpFileList() != null) {
            for (FileBean fileBean : waterMeterEntities.get(page).getBpFileList()) {
                files1.add(new File(fileBean.getFilePath()));
            }
        }
        map.put("bpFileList", files1);
        List<File> files2 = new ArrayList<>();
        if (waterMeterEntities.get(page).getExceptionFileList() != null) {
            for (FileBean fileBean : waterMeterEntities.get(page).getExceptionFileList()) {
                files2.add(new File(fileBean.getFilePath()));
            }
        }
        map.put("exceptionFileList", files2);

        List<File> files3 = new ArrayList<>();
        if (waterMeterEntities.get(page).getWfbdFileList() != null) {
            for (FileBean fileBean : waterMeterEntities.get(page).getWfbdFileList()) {
                files3.add(new File(fileBean.getFilePath()));
            }
        }
        map.put("wfbdFileList", files3);
        mPresenter.getResultThree(requireContext(), map);
    }

    private void initRecyclerView() {
        adapter = new DataUpdateAdapter(list);
        RecyclerViewUtils.initLinerV(requireContext(), binding.rvList, adapter, true, 1);
        adapter.setOnItemClickListener((adapter, view, position) -> {
            Intent intent = null;
            switch (list.get(position).getType()) {
                case 0:
                case 1:
                case 2:
                    intent = new Intent(requireContext(), TaskDetailActivity.class);
                    intent.putExtra("Id", list.get(position).getId());
                    intent.putExtra("type", list.get(position).getType());
                    startActivity(intent);
                    break;
                default:
                    intent = new Intent(requireContext(), UnRegisterActivity.class);
                    intent.putExtra("Id", list.get(position).getId());
                    startActivity(intent);
                    break;
            }
        });
        List<String> strings = Arrays.asList("全部", "正常", "异常", "无法标定", "非在册");
        SpinnerUtils.initSpinner(requireContext(), binding.spinnerType, strings);
        binding.spinnerType.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                type = position - 1;
                initList();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    private void initList() {
        list.clear();
        binding.tvNumber.setText(String.valueOf(DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().queryCountByQueryBuilder()));
        if (type >= 0) {
            list.addAll(DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().queryByQueryBuilder(BaseUpdataEntityDao.Properties.Type.eq(type)));
        } else {
            list.addAll(DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().queryAll());
        }
        adapter.setNewData(list);
    }


    @Override
    public <T> void getResultOne(T response) {
        BaseResponse baseResponse = (BaseResponse) response;
        if (baseResponse != null && "200".equals(baseResponse.getCode())) {
            DaoUtilsStore.getInstance().getSbdjEntityDbUtils().delete(sbdjEntities.get(page));
            for (BaseUpdataEntity entity : list) {
                if (entity.getId().equals(sbdjEntities.get(page).getId())) {
                    DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().delete(entity);
                }
            }
            page++;
            if (page <= sbdjEntities.size() - 1) {
                updataSbdj();
            } else {
                if (twdhEntities != null && twdhEntities.size() > 0) {
                    page = 0;
                    updataTwdh();
                } else if (waterMeterEntities != null && waterMeterEntities.size() > 0) {
                    page = 0;
                    updataWaterMeter();
                } else {
                    hideLoading();
                    initList();
                    onMessage("数据上传完毕");
                }
            }
        } else {
            hideLoading();
            onMessage(baseResponse.getMessage());
        }
    }

    @Override
    public <T> void getResultTwo(T response) {
        BaseResponse baseResponse = (BaseResponse) response;
        if (baseResponse != null && "200".equals(baseResponse.getCode())) {
            twdhEntities.get(page).setState(null);
            String[] id = twdhEntities.get(page).getWaterNavigationInspects().split(",");
            for (String str : id) {
                if (!TextUtils.isEmpty(str)) {
                    List<TwdhEntity> twdhEntityList = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilder(TwdhEntityDao.Properties.WaterNavigationInspects.like("%" + str + "%"));
                    for (TwdhEntity entity1 : twdhEntityList) {
                        String waterNavigationInspects = entity1.getWaterNavigationInspects().replace("," + str, "").replace(str, "");
                        entity1.setWaterNavigationInspects(waterNavigationInspects);
                        if (TextUtils.isEmpty(entity1.getWaterNavigationInspects())) {
                            DaoUtilsStore.getInstance().getTwdhEntityDbUtils().delete(entity1);
                        } else {
                            DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertOrReplace(entity1);
                        }
                    }
                }
            }
            DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertOrReplace(twdhEntities.get(page));
            page++;
            if (page <= twdhEntities.size() - 1) {
                updataTwdh();
            } else {
                if (waterMeterEntities != null && waterMeterEntities.size() > 0) {
                    page = 0;
                    updataWaterMeter();
                } else {
                    hideLoading();
                    initList();
                    onMessage("数据上传完毕");
                }
            }
        } else {
            hideLoading();
            onMessage(baseResponse.getMessage());
        }
    }

    @Override
    public <T> void getResultThree(T response) {
        BaseResponse baseResponse = (BaseResponse) response;
        if (baseResponse != null && "200".equals(baseResponse.getCode())) {
            waterMeterEntities.get(page).setState(3);
            DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().insertOrReplace(waterMeterEntities.get(page));
            for (BaseUpdataEntity entity : list) {
                if (entity.getId().equals(waterMeterEntities.get(page).getId())) {
                    DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().delete(entity);
                }
            }
            page++;
            if (page <= waterMeterEntities.size() - 1) {
                updataWaterMeter();
            } else {
                hideLoading();
                List<WaterMeterTaskEntity> waterMeterTaskEntities = DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().queryByQueryBuilder(WaterMeterTaskEntityDao.Properties.CheckUser.eq(AccountManager.getUserId()));
                for (WaterMeterTaskEntity entity : waterMeterTaskEntities) {
                    boolean flag = true;
                    for (WaterMeterEntity waterMeter : entity.getYxswWaterMeterInspectDto()) {
                        WaterMeterEntity dbWaterMeter = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryById(waterMeter.getId());
                        waterMeter.setState(dbWaterMeter.getState());
                        if (dbWaterMeter.getState() != 3) {
                            flag = false;
                        }
                    }
                    if (flag) {
                        entity.setState("2");
                        DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().insertOrReplace(entity);
                    }
                }
                initList();
                onMessage("数据上传完毕");
            }
        } else {
            hideLoading();
            onMessage(baseResponse.getMessage());
        }
    }
}

