package com.tofly.yxpc.ui.fragment;

import android.content.Intent;
import android.text.TextUtils;
import android.view.View;

import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.base.LatteDelegate;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.databinding.FragmentIndexBinding;
import com.tofly.yxpc.entity.TwdhEntity;
import com.tofly.yxpc.entity.WaterMeterEntity;
import com.tofly.yxpc.entity.WaterMeterTaskEntity;
import com.tofly.yxpc.greenDao.DictEntityDao;
import com.tofly.yxpc.greenDao.TwdhEntityDao;
import com.tofly.yxpc.greenDao.WaterMeterEntityDao;
import com.tofly.yxpc.greenDao.WaterMeterTaskEntityDao;
import com.tofly.yxpc.ui.activity.ImageTextNavitActivity;
import com.tofly.yxpc.ui.activity.TaskActivity;
import com.tofly.yxpc.ui.activity.UnRegisterActivity;
import com.tofly.yxpc.ui.adapter.IndexTaskAdapter;
import com.tofly.yxpc.ui.impl.DownLoadImpl;
import com.tofly.yxpc.utils.RecyclerViewUtils;
import com.tofly.yxpc.utils.db.DaoUtilsStore;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ychk
 */
public class IndexFragment extends LatteDelegate {

    private FragmentIndexBinding binding;
    private IndexTaskAdapter taskAdapter;
    private final List<WaterMeterTaskEntity> list = new ArrayList<>();
    private OnClickListener listener = null;
    private boolean isFirstLoading = true;

    public IndexFragment(OnClickListener listener) {
        this.listener = listener;
    }

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new DownLoadImpl();
    }

    @Override
    public Object setLayout() {
        binding = FragmentIndexBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    public void initView() {
        initRecyclerView();
        initData();
        initListener();
    }

    @Override
    public void onResume() {
        super.onResume();
        if (!isFirstLoading) {
            //如果不是第一次加载，刷新数据
            initData();
        }
        isFirstLoading = false;
        binding.scrollview.scrollTo(0, 0);
    }

    private void initRecyclerView() {
        taskAdapter = new IndexTaskAdapter(list);
        RecyclerViewUtils.initLinerV(requireContext(), binding.rvListTask, taskAdapter, false, 0);
    }

    private void initData() {
        list.clear();
        new Thread(() -> {
            int total = 0;
            int wwc = 0;
            int wsc = 0;
            List<WaterMeterTaskEntity> waterMeterTaskEntities = DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().queryByQueryBuilder(WaterMeterTaskEntityDao.Properties.CheckUser.eq(AccountManager.getUserId()), WaterMeterTaskEntityDao.Properties.State.in("0", "1"));
            for (WaterMeterTaskEntity entity : waterMeterTaskEntities) {
                total += (int) DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryCountByQueryBuilder(WaterMeterEntityDao.Properties.TaskId.eq(entity.getId()));
                wwc += (int) DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryCountByQueryBuilder(WaterMeterEntityDao.Properties.TaskId.eq(entity.getId()), WaterMeterEntityDao.Properties.State.in(0, 1));
                wsc += (int) DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryCountByQueryBuilder(WaterMeterEntityDao.Properties.TaskId.eq(entity.getId()), WaterMeterEntityDao.Properties.State.eq(2));
            }
            int finalWsc = wsc;
            int finalTotal = total;
            int finalWwc = wwc;

            waterMeterTaskEntities = DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().queryByQueryBuilder(WaterMeterTaskEntityDao.Properties.CheckUser.eq(AccountManager.getUserId()));
            for (WaterMeterTaskEntity entity : waterMeterTaskEntities) {
                Long count = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryCountByQueryBuilder(WaterMeterEntityDao.Properties.TaskId.eq(entity.getId()));
                if (count == 0) {
                    DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().delete(entity);
                }
            }
            list.addAll(DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().queryByQueryBuilder(WaterMeterTaskEntityDao.Properties.CheckUser.eq(AccountManager.getUserId()), WaterMeterTaskEntityDao.Properties.State.notEq("2")));

            getActivity().runOnUiThread(() -> {
                binding.tvTotalNumber.setText(String.valueOf(finalTotal));
                binding.tvCompleteNumber.setText(String.valueOf(finalTotal - finalWwc));
                binding.tvDealtNumber.setText(String.valueOf(finalWwc));
                if (finalWsc > 0) {
                    binding.tvUpdateNumberNo.setText("待上传" + finalWsc + "个");
                    binding.tvUpdateNumberNo.setVisibility(View.VISIBLE);
                } else {
                    binding.tvUpdateNumberNo.setVisibility(View.GONE);
                }
                if (finalTotal != 0) {
                    binding.pbComplete.setProgress((finalTotal - finalWwc) * 100 / finalTotal);
                    binding.tvProgress.setText((finalTotal - finalWwc) * 100 / finalTotal + "%");
                } else {
                    binding.pbComplete.setProgress(0);
                }
                taskAdapter.setNewData(list);
            });
        }).start();
    }

    private void initListener() {
        binding.llNavit.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime()) {
                Intent intent = new Intent(requireActivity(), ImageTextNavitActivity.class);
                startActivity(intent);
            }
        });
        binding.llUnregister.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime()) {
                Long count = DaoUtilsStore.getInstance().getDictEntityDbUtils().queryCountByQueryBuilder(DictEntityDao.Properties.DictCode.eq("noregistered_type"));
                if (count > 0) {
                    Intent intent = new Intent(requireActivity(), UnRegisterActivity.class);
                    startActivity(intent);
                } else {
                    onMessage("基础数据加载失败");
                }
            }
        });
        binding.llDataUpdate.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime() && listener != null) {
                listener.toFragment();
            }
        });
        binding.llDataDown.setOnClickListener(v -> {
            if (NetworkUtil.isNetworkAvailable()) {
                getData();
            } else {
                onMessage("暂无网络，不可下载！");
            }
        });
        taskAdapter.setOnItemClickListener((adapter, view, position) -> {
            Intent intent = new Intent(requireActivity(), TaskActivity.class);
            intent.putExtra("Id", list.get(position).getId());
            startActivity(intent);
        });
    }

    /**
     * 下载任务
     */
    private void getData() {
        Map<String, Object> m1 = new HashMap<>(4);
        m1.put("checkUser", AccountManager.getUserId());
        mPresenter.getResultTwo(requireContext(), m1);

        showLoading("任务下载中...");
        Map<String, Object> map = new HashMap<>();
        map.put("userId", AccountManager.getUserId());
        mPresenter.getResultOne(requireContext(), map);
    }

    public interface OnClickListener {
        void toFragment();
    }

    @Override
    public <T> void getResultOne(T response) {
        hideLoading();
        if (response != null) {
            DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().delete(DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().queryByQueryBuilder(WaterMeterTaskEntityDao.Properties.CheckUser.eq(AccountManager.getUserId())));
            List<WaterMeterTaskEntity> waterMeterTaskEntities = (List<WaterMeterTaskEntity>) response;
            DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().insertMulti(waterMeterTaskEntities, true);
            for (WaterMeterTaskEntity waterMeterTaskEntity : waterMeterTaskEntities) {
                if (waterMeterTaskEntity.getYxswWaterMeterInspectDto() != null && waterMeterTaskEntity.getYxswWaterMeterInspectDto().size() > 0) {
                    for (WaterMeterEntity sbdjEntity : waterMeterTaskEntity.getYxswWaterMeterInspectDto()) {
                        sbdjEntity.setTaskId(waterMeterTaskEntity.getId());
                        sbdjEntity.setCheckUser(waterMeterTaskEntity.getCheckUser());
                        WaterMeterEntity dbWaterMeter = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryById(sbdjEntity.getId());
                        if (dbWaterMeter == null || dbWaterMeter.getState() == 0 || dbWaterMeter.getState() == 3) {
                            DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().insertOrReplace(sbdjEntity);
                        }
                    }
                }
            }
            list.clear();
            initData();
            onMessage("任务下载成功");
        }
    }

    @Override
    public <T> void getResultTwo(T response) {
        if (response != null) {
            List<TwdhEntity> twdhEntities = (List<TwdhEntity>) response;
            if (!twdhEntities.isEmpty()) {
                List<TwdhEntity> twdhEntityList = new ArrayList<>(twdhEntities);
                for (int i = 0; i < twdhEntityList.size(); i++) {
                    String[] ids = twdhEntityList.get(i).getWaterNavigationInspects().split(",");
                    for (String s : ids) {
                        if (!TextUtils.isEmpty(s)) {
                            TwdhEntity twdhEntity = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilderOne(TwdhEntityDao.Properties.WaterNavigationInspects.like("%" + s + "%"));
                            if (twdhEntity != null) {
                                twdhEntities.remove(twdhEntityList.get(i));
                                break;
                            }
                        }
                    }
                }
                DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertMulti(twdhEntities, false);
            }
        }
    }
}
