package com.tofly.yxpc.ui.fragment;

import android.content.Intent;
import android.text.TextUtils;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.RequestOptions;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.base.LatteDelegate;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.base.NetWorkUrl;
import com.tofly.yxpc.ui.activity.AboutSystemActivity;
import com.tofly.yxpc.ui.activity.LoginActivity;
import com.tofly.yxpc.ui.activity.SettingActivity;
import com.tofly.yxpc.ui.impl.MyImpl;
import com.tofly.yxpc.utils.DialogUtils;
import com.tofly.yxpc.widget.CircleImageView;

import butterknife.BindView;
import butterknife.OnClick;

/**
 * @author ychk
 */
public class MeFragment extends LatteDelegate {

    @BindView(R.id.cimg_head_portrait)
    CircleImageView cimgHeadPortrait;
    @BindView(R.id.ll_head_portrait)
    LinearLayout llHeadPortrait;
    @BindView(R.id.tv_user_name)
    TextView tvUserName;
    @BindView(R.id.ll_about_system)
    LinearLayout llAboutSystem;
    @BindView(R.id.tv_login_out)
    TextView tvLoginOut;

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new MyImpl();
    }

    @Override
    public Object setLayout() {
        return R.layout.fragment_my;
    }

    @Override
    public void initView() {
    }

    @Override
    public void onResume() {
        super.onResume();
        loadData();
    }

    @OnClick({R.id.ll_head_portrait, R.id.ll_about_system, R.id.tv_login_out})
    public void onClick(View view) {
        if (!MyApp.getInstance().isOverTime()) {
            return;
        }
        Intent intent = null;
        switch (view.getId()) {
            case R.id.ll_head_portrait:
                intent = new Intent(requireContext(), SettingActivity.class);
                requireContext().startActivity(intent);
                break;
            case R.id.ll_about_system:
                intent = new Intent(requireContext(), AboutSystemActivity.class);
                requireContext().startActivity(intent);
                break;
            case R.id.tv_login_out:
                DialogUtils.showDialog(requireContext(), "是否确定退出", "", true, (dialog, which) -> {
                    requireActivity().finish();
                    startActivity(new Intent(getContext(), LoginActivity.class));
                });
                break;
            default:
                break;
        }
    }

    /**
     * 加载基础信息
     */
    private void loadData() {
        if (!TextUtils.isEmpty(AccountManager.getAvatar())) {
            Glide.with(requireContext())
                    .load(NetWorkUrl.getFileUrl(AccountManager.getAvatar()))
                    .apply(new RequestOptions().error(R.mipmap.icon_head_portrait))
                    .into(cimgHeadPortrait);
        } else {
            cimgHeadPortrait.setBackgroundResource(R.mipmap.icon_head_portrait);
        }
        tvUserName.setText(AccountManager.getUserRealName() == null ? "" : AccountManager.getUserRealName());
    }

    @Override
    public <T> void getResultOne(T response) {

    }
}
