package com.tofly.yxpc.ui.fragment;

import static android.app.Activity.RESULT_OK;
import static com.tofly.latte_core.config.Constants.PHOTO_PATH;

import android.Manifest;
import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Rect;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.MediaStore;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextPaint;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.WindowManager;

import androidx.annotation.CallSuper;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.content.FileProvider;

import com.amap.api.maps.AMap;
import com.amap.api.maps.CameraUpdate;
import com.amap.api.maps.CameraUpdateFactory;
import com.amap.api.maps.MapView;
import com.amap.api.maps.UiSettings;
import com.amap.api.maps.model.BitmapDescriptorFactory;
import com.amap.api.maps.model.CameraPosition;
import com.amap.api.maps.model.LatLng;
import com.amap.api.maps.model.MarkerOptions;
import com.amap.api.services.core.LatLonPoint;
import com.amap.api.services.geocoder.GeocodeResult;
import com.amap.api.services.geocoder.GeocodeSearch;
import com.amap.api.services.geocoder.RegeocodeQuery;
import com.amap.api.services.geocoder.RegeocodeResult;
import com.jph.takephoto.app.TakePhoto;
import com.jph.takephoto.app.TakePhotoImpl;
import com.jph.takephoto.compress.CompressConfig;
import com.jph.takephoto.model.InvokeParam;
import com.jph.takephoto.model.TContextWrap;
import com.jph.takephoto.model.TImage;
import com.jph.takephoto.model.TResult;
import com.jph.takephoto.permission.InvokeListener;
import com.jph.takephoto.permission.PermissionManager;
import com.jph.takephoto.permission.TakePhotoInvocationHandler;
import com.tofly.latte_core.base.LatteDelegate;
import com.tofly.latte_core.config.Constants;
import com.tofly.latte_core.config.Latte;
import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.entity.user.UserBean;
import com.tofly.yxpc.map.LocationUtil_xu;
import com.tofly.yxpc.map.gaode.AmapLocationHelper;
import com.tofly.yxpc.utils.BitmapUtils;
import com.tofly.yxpc.utils.DateUtils;
import com.tofly.yxpc.utils.WaterMarkParams;
import com.tofly.yxpc.utils.video.VideoProcessorUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import cn.hzw.graffiti.GraffitiActivity;
import cn.hzw.graffiti.GraffitiParams;

/**
 * @author ychk
 */
public abstract class PhotoFragment extends LatteDelegate implements TakePhoto.TakeResultListener, InvokeListener {

    protected TakePhoto takePhoto;
    private InvokeParam invokeParam;
    public static final int REQ_CODE_DOODLE = 1081;
    public AmapLocationHelper mapLoadHelper;
    protected LocationUtil_xu.CoordinateClass lo = null;
    private boolean isWaterMark = true;
    public AMap aMap;
    public MapView mapView;
    private boolean isLocation = true;
    private boolean isRed = true;
    public static final int VIDEO_REQUEST_CODE = 66;
    private boolean flag = true;
    private boolean isFirst = true;

    @CallSuper
    @Override
    public void initView() {
        getTakePhoto().onCreate(savedInstanceState);
        takePhoto.onEnableCompress(getCompressConfig(), true);
        mapLoadHelper = new AmapLocationHelper();
        mapLoadHelper.startLocating();
        initMap(savedInstanceState);
        mapLoadHelper.setMapLocationListener((aMpPoint, addr, arcgisPoint) -> {
            lo = new LocationUtil_xu.CoordinateClass(arcgisPoint.longitude, arcgisPoint.latitude, addr);
            if (TextUtils.isEmpty(addr)) {
                mapLoadHelper.resetLocation();
            }
            if (isFirst) {
                isFirst = false;
                finishLocation();
            }
        });
    }

    /**
     * 获取TakePhoto实例
     */
    public TakePhoto getTakePhoto() {
        if (takePhoto == null) {
            takePhoto = (TakePhoto) TakePhotoInvocationHandler.of(this).bind(new TakePhotoImpl(this, this));
        }
        return takePhoto;
    }

    /**
     * 设置是否自动定位
     */
    public void setIsLocation(boolean isLocation) {
        this.isLocation = isLocation;
    }

    /**
     * 设置定位图标
     */
    public void setIsRed(boolean isRed) {
        this.isRed = isRed;
    }

    private void initMap(Bundle savedInstanceState) {
        mapView = rootView.findViewById(R.id.map_view);
        if (mapView != null) {
            mapView.onCreate(savedInstanceState);
            if (aMap == null) {
                aMap = mapView.getMap();
                aMap.moveCamera(CameraUpdateFactory.zoomBy(1));
                aMap.getUiSettings().setMyLocationButtonEnabled(false);
                if (isLocation) {
                    aMap.setMyLocationEnabled(true);
                    aMap.setMyLocationType(AMap.LOCATION_TYPE_LOCATE);
                } else {
                    aMap.setMyLocationEnabled(false);
                }
                UiSettings uiSettings = aMap.getUiSettings();
                uiSettings.setScaleControlsEnabled(true);
                uiSettings.setZoomControlsEnabled(false);
            }
            initMapView();
        }
    }

    public void initMapView() {

    }

    public void checkMap() {
        if (!flag) {
            aMap.setMapType(AMap.MAP_TYPE_SATELLITE);
        } else {
            aMap.setMapType(AMap.MAP_TYPE_NORMAL);
        }
        flag = !flag;
    }

    /**
     * 地图点击监听事件
     */

    public AMap.OnMapClickListener mapClickListener = latLng -> {
        LocationUtil_xu.CoordinateClass coordinate = LocationUtil_xu.gcj02towgs84(latLng.longitude, latLng.latitude);
        if (coordinate == null) {
            return;
        }
        double lng = LocationUtil_xu.formatLocationDouble(coordinate.Longtitude);
        double lat = LocationUtil_xu.formatLocationDouble(coordinate.Latitude);
        LatLng mP = new LatLng(lat, lng);
        aMap.clear();
        aMap.addMarker(setMark(latLng, isRed));
        getAddress(latLng, mP);
    };
    public AMap.OnCameraChangeListener mapCameraChangeListener = new AMap.OnCameraChangeListener() {
        @Override
        public void onCameraChange(CameraPosition cameraPosition) {

        }

        @Override
        public void onCameraChangeFinish(CameraPosition cameraPosition) {
            double latitude = cameraPosition.target.latitude;
            double longitude = cameraPosition.target.longitude;
            LatLng latLng = new LatLng(latitude, longitude);
            LocationUtil_xu.CoordinateClass coordinate = LocationUtil_xu.gcj02towgs84(longitude, latitude);
            if (coordinate == null) {
                return;
            }
            double lng = LocationUtil_xu.formatLocationDouble(coordinate.Longtitude);
            double lat = LocationUtil_xu.formatLocationDouble(coordinate.Latitude);
            LatLng mP = new LatLng(lat, lng);
            getAddress(latLng, mP);
        }
    };

    public void initMapListener(LocationUtil_xu.CoordinateClass info) {

    }

    /**
     * 将经纬度转化为地理位置
     *
     * @param latLng 经纬度
     */
    private void getAddress(LatLng latLng, LatLng gisPoint) {
        GeocodeSearch geocodeSearch = new GeocodeSearch(requireContext());
        LatLonPoint lp = new LatLonPoint(latLng.latitude, latLng.longitude);
        RegeocodeQuery query = new RegeocodeQuery(lp, 10, GeocodeSearch.AMAP);
        geocodeSearch.getFromLocationAsyn(query);
        geocodeSearch.setOnGeocodeSearchListener(new GeocodeSearch.OnGeocodeSearchListener() {
            @Override
            public void onRegeocodeSearched(RegeocodeResult regeocodeResult, int i) {
                LocationUtil_xu.CoordinateClass loNew = new LocationUtil_xu.CoordinateClass(gisPoint.longitude, gisPoint.latitude, regeocodeResult.getRegeocodeAddress().getFormatAddress());
                initMapListener(loNew);
            }

            @Override
            public void onGeocodeSearched(GeocodeResult geocodeResult, int i) {

            }
        });
    }

    /**
     * 设置压缩参数
     */
    public CompressConfig getCompressConfig() {
        //设置压缩参数
        return new CompressConfig.Builder().setMaxSize(50 * 1024).setMaxPixel(800).create();
    }

    /**
     * 打开文件夹
     */
    public void showFileChooser() {
        Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
        intent.setType("*/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        try {
            startActivityForResult(Intent.createChooser(intent, "文件路径"), 1);
        } catch (ActivityNotFoundException ex) {
            ex.printStackTrace();
        }
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        getTakePhoto().onSaveInstanceState(outState);
        super.onSaveInstanceState(outState);
    }

    @Override
    public PermissionManager.TPermissionType invoke(InvokeParam invokeParam) {
        PermissionManager.TPermissionType type = PermissionManager.checkPermission(TContextWrap.of(this), invokeParam.getMethod());
        if (PermissionManager.TPermissionType.WAIT.equals(type)) {
            this.invokeParam = invokeParam;
        }
        return type;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        //以下代码为处理Android6.0、7.0动态权限所需
        PermissionManager.TPermissionType type = PermissionManager.onRequestPermissionsResult(requestCode, permissions, grantResults);
        PermissionManager.handlePermissionsResult(requireActivity(), type, invokeParam, this);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == REQ_CODE_DOODLE) {
            if (resultCode == RESULT_OK) {
                if (data != null) {
                    String imgUrl = data.getStringExtra(GraffitiActivity.KEY_IMAGE_PATH);
                    BitmapUtils.compressImage(imgUrl);
                    onTakePhotoSuccess(imgUrl);
                } else {
                    onMessage("获取图片失败");
                }
            } else {
                onMessage("获取图片失败");
            }
        } else if (requestCode == VIDEO_REQUEST_CODE && resultCode == RESULT_OK) {
            if (data != null) {
                Uri uri = data.getData();
                if (uri != null) {
                    String oldPath = uri.toString().replace("content://com.tofly.yxpc.fileprovider/camera_photos", "");
                    showLoading("视频压缩中...");
                    VideoProcessorUtils.compressVideo2(requireContext(), oldPath, msg -> {
                        if (msg.obj != null) {
                            requireActivity().runOnUiThread(() -> {
                                onTakeVideoSuccess((String) msg.obj);
                                hideLoading();
                            });
                        }
                        return false;
                    });
                }
            }
        } else {
            getTakePhoto().onActivityResult(requestCode, resultCode, data);
        }
    }

    public abstract void onTakePhotoSuccess(String pathFile);

    public abstract void onTakeVideoSuccess(String videoFile);

    @Override
    public void takeSuccess(TResult result) {
        try {
            //获取照片成功
            String originalPath = result.getImage().getOriginalPath();
            if (isWaterMark) {
                //通过路径获取图片,将图片转换成bitmap.在通过对bitmap进行添加水印
                Bitmap bitmap = BitmapFactory.decodeFile(originalPath);
                WaterMarkParams waterMarkParams = getWaterMarkParams();
                String fistLine = "经度:" + " 纬度:";
                String secLine = "地址:";
                if (waterMarkParams != null) {
                    fistLine = "经度:" + waterMarkParams.mLongitude + " 纬度:" + waterMarkParams.mLatitude;
                    secLine = "地址:" + waterMarkParams.mAddress;
                }
                String thirdLine = "上报时间:" + DateUtils.getCurrentDate();
                String mark = fistLine + "\n" + secLine + "\n" + thirdLine;
                Bitmap watermark = createWatermark(bitmap, mark);
                File file = saveImage(watermark, originalPath);
                String absolutePath = file.getAbsolutePath();
                TImage tImage = result.getImage();
                tImage.setOriginalPath(absolutePath);
            }
            //  takeGraffiti(result.getImage().getOriginalPath());
            String imgUrl = result.getImage().getOriginalPath();
            BitmapUtils.compressImage(imgUrl);
            onTakePhotoSuccess(imgUrl);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void takeCancel() {

    }

    /**
     * 自动定位回调
     */
    public void finishLocation() {

    }

    public static File saveImage(Bitmap bmp, String oldPath) {
        File file;
        if (oldPath.contains(PHOTO_PATH)) {
            file = new File(oldPath);
        } else {
            File appDir = new File(Constants.PHOTO_PATH);
            if (!appDir.isDirectory()) {
                appDir.mkdirs();
            }
            String fileName = System.currentTimeMillis() + ".jpg";
            file = new File(appDir, fileName);
        }
        try {
            FileOutputStream fos = new FileOutputStream(file);
            bmp.compress(Bitmap.CompressFormat.JPEG, 100, fos);
            fos.flush();
            fos.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return file;
    }

    protected WaterMarkParams getWaterMarkParams() {
        WaterMarkParams params = null;
        if (lo != null) {
            params = new WaterMarkParams();
            params.mbNeedGps = 1;
            String strLng = LocationUtil_xu.formatLocation(lo.Longtitude);
            String strLat = LocationUtil_xu.formatLocation(lo.Latitude);
            if (!strLng.isEmpty() && !strLat.isEmpty()) {
                double lng = Double.parseDouble(strLng);
                double lat = Double.parseDouble(strLat);
                strLng = LocationUtil_xu.formatLocation(lng);
                strLat = LocationUtil_xu.formatLocation(lat);
                params.mLongitude = strLng;
                params.mLatitude = strLat;
            }
            params.mAddress = lo.addr;
            UserBean userBean = new UserBean();
            params.mUser = userBean.getUsername();
            params.mReportTime = DateUtils.getDate();
        }
        return params;
    }

    public Bitmap createWatermark(Bitmap bitmapS, String markText) {
        // 获取图片的宽高
        int bitmapWidth = bitmapS.getWidth();
        int bitmapHeight = bitmapS.getHeight();
        // 创建一个和图片一样大的背景图
        Bitmap bmp = Bitmap.createBitmap(bitmapWidth, bitmapHeight, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bmp);
        // 画背景图
        canvas.drawBitmap(bitmapS, 0, 0, null);
        //-------------开始绘制文字-------------------------------
        if (!TextUtils.isEmpty(markText)) {
            int screenWidth = getScreenWidth();
            float textSize = (float) (MyApp.dip2px(12) * bitmapWidth) / screenWidth;
            // 创建画笔
            TextPaint mPaint = new TextPaint();
            // 文字矩阵区域
            Rect textBounds = new Rect();
            // 水印的字体大小
            mPaint.setTextSize(textSize);
            // 文字阴影
            mPaint.setShadowLayer(0.5f, 0f, 1f, Color.YELLOW);
            // 抗锯齿
            mPaint.setAntiAlias(true);
            // 水印的区域
            mPaint.getTextBounds(markText, 0, markText.length(), textBounds);
            // 水印的颜色
            mPaint.setColor(Color.WHITE);
            StaticLayout layout = new StaticLayout(markText, 0, markText.length(), mPaint, (int) (bitmapWidth - textSize),
                    Layout.Alignment.ALIGN_NORMAL, 1.0F, 0.5F, true);
            // 文字开始的坐标
            float textX = (float) bitmapHeight / 40;
            //图片的中间
            float textY = (float) (bitmapHeight * 7) / 9;
            // 画文字
            canvas.translate(textX, textY);
            layout.draw(canvas);
        }
        //保存所有元素
        canvas.save();
        canvas.restore();
        return bmp;
    }

    /**
     * 图片涂鸦
     */
    public void takeGraffiti(String pathFile) {
        File file = new File(pathFile);
        String filePath = Constants.PHOTO_PATH + file.getName();
        // 涂鸦参数
        GraffitiParams params = new GraffitiParams();
        params.mIsFullScreen = true;
        // 图片路径
        params.mImagePath = pathFile;
        //图片保存路径
        params.mSavePath = filePath;
        params.mPaintSize = 15f;
        // 启动涂鸦页面
        Intent intent = new Intent(requireContext(), GraffitiActivity.class);
        intent.putExtra(GraffitiActivity.KEY_PARAMS, params);
        startActivityForResult(intent, REQ_CODE_DOODLE);
    }

    private int getScreenWidth() {
        DisplayMetrics dm = new DisplayMetrics();
        WindowManager wm = (WindowManager) requireActivity().getSystemService(Context.WINDOW_SERVICE);
        wm.getDefaultDisplay().getMetrics(dm);
        return dm.widthPixels;
    }

    @Override
    public void takeFail(TResult result, String msg) {
        ToastUtils.showToast("拍照或相册失败，请重试");
    }

    /**
     * 默认拍照地址
     */
    public Uri getImageUrl() {
        File appDir = new File(Constants.PHOTO_PATH);
        if (!appDir.isDirectory()) {
            appDir.mkdirs();
        }
        File file = new File(Constants.PHOTO_PATH + System.currentTimeMillis() + ".jpg");
        return Uri.fromFile(file);
    }

    public void showPhoto(boolean isSingle, boolean isWaterMark) {
        this.isWaterMark = isWaterMark;
        takePhoto(isSingle);
    }

    public void showPhoto(boolean isSingle) {
        isWaterMark = true;
        takePhoto(isSingle);
    }

    private void takePhoto(boolean isSingle) {
        if (isSingle) {
            takePhoto.onPickFromCapture(getImageUrl());
        } else {
            final String[] items = {"拍照", "相册", "取消"};
            new AlertDialog.Builder(requireContext())
                    .setItems(items, (dialog, which) -> {
                        switch (which) {
                            case 0:
                                takePhoto.onPickFromCapture(getImageUrl());
                                break;
                            case 1:
                                takePhoto.onPickFromGallery();
                                break;
                            case 2:
                                dialog.dismiss();
                                break;
                            default:
                                break;
                        }
                    }).setCancelable(true)
                    .show();
        }
    }

    public void showVideo(boolean isWaterMark) {
        this.isWaterMark = isWaterMark;
        takeVideo();
    }

    private void takeVideo() {
        final String[] items = {"拍照", "相册", "视频", "取消"};
        new AlertDialog.Builder(requireContext())
                .setItems(items, (dialog, which) -> {
                    switch (which) {
                        case 0:
                            takePhoto.onPickFromCapture(getImageUrl());
                            break;
                        case 1:
                            takePhoto.onPickFromGallery();
                            break;
                        case 2:
                            getVideo();
                            break;
                        default:
                            dialog.dismiss();
                            break;
                    }
                }).setCancelable(true)
                .show();
    }

    private void getVideo() {
        checkPermission(isSuccess -> {
                    if (isSuccess) {
                        try {
                            File file = new File(Constants.VIDEO_PATH + System.currentTimeMillis() + ".mp4");
                            File parentFile = file.getParentFile();
                            if (!parentFile.exists()) {
                                boolean mkdirs = parentFile.mkdirs();
                                Log.e("TAG", "video: 00000000000000000" + mkdirs);
                            }
                            if (file.exists()) {
                                file.delete();
                            }
                            file.createNewFile();
                            Intent intent = new Intent(MediaStore.ACTION_VIDEO_CAPTURE);
                            Uri videoUri;
                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                                intent.setFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
                                videoUri = FileProvider.getUriForFile(requireContext(), Latte.getApplicationContext().getPackageName() + ".fileprovider", file);
                            } else {
                                videoUri = Uri.fromFile(file);
                            }
                            //储存地址
                            intent.putExtra(MediaStore.EXTRA_OUTPUT, videoUri);
                            //用于控制录制视频的质量；0——低质量；1——高质量
                            intent.putExtra(MediaStore.EXTRA_VIDEO_QUALITY, 1);
                            //允许记录的最长时间(以 秒 为单位)  例如：限制为60S
                            intent.putExtra(MediaStore.EXTRA_DURATION_LIMIT, 10);
                            //允许的最大大小(以 B 为单位)  例如：限制大小为100M
                            intent.putExtra(MediaStore.EXTRA_SIZE_LIMIT, 1024 * 1024 * 80);
                            Activity activity = requireActivity();
                            activity.startActivityForResult(intent, VIDEO_REQUEST_CODE);
                        } catch (Exception e) {
                            Log.e("TAG", e.toString());
                            onMessage("启用相机失败，请重试");
                        }
                    } else {
                        onMessage("请打开权限");
                    }
                }, Manifest.permission.CAMERA,
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.RECORD_AUDIO);
    }

    /**
     * 绘制单个点(使用wgs84的点绘制)
     */
    public MarkerOptions setMark(Double log, Double lat, String type) {
        LatLng markPoint = LocationUtil_xu.wgs84togcj02(requireContext(), log, lat);
        MarkerOptions options = new MarkerOptions();
        if (markPoint != null) {
            options.position(markPoint);
        }
        options.draggable(false);
        switch (type) {
            case "0":
                options.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory.decodeResource(getResources(), R.mipmap.icon_location_red)));
                break;
            case "1":
                options.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory.decodeResource(getResources(), R.mipmap.icon_location_my)));
                break;
            case "2":
                options.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory.decodeResource(getResources(), R.mipmap.icon_location_blue)));
                break;
            case "3":
                options.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory.decodeResource(getResources(), R.mipmap.icon_location_drag)));
                break;
        }
        options.setFlat(false);
        return options;
    }

    /**
     * 高德的点绘制
     */
    public MarkerOptions setMark(LatLng latLng, boolean isRed) {
        MarkerOptions options = new MarkerOptions();
        if (latLng != null) {
            options.position(latLng);
        }
        options.draggable(false);
        if (isRed) {
            options.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory.decodeResource(getResources(), R.mipmap.icon_location_red)));
        } else {
            options.icon(BitmapDescriptorFactory.fromBitmap(BitmapFactory.decodeResource(getResources(), R.mipmap.icon_location_drag)));
        }
        options.setFlat(false);
        return options;
    }

    /**
     * 移动到固定点位置
     */
    public void moveOrigin(Double log, Double lat, float zoomLevel) {
        LatLng movePoint = LocationUtil_xu.wgs84togcj02(requireContext(), log, lat);
        CameraPosition cameraPosition = new CameraPosition(movePoint, zoomLevel, 0, 30);
        CameraUpdate cameraUpdate = CameraUpdateFactory.newCameraPosition(cameraPosition);
        aMap.moveCamera(cameraUpdate);
    }

    @Override
    public void onResume() {
        super.onResume();
        if (mapView != null) {
            mapView.onResume();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if (mapView != null) {
            mapView.onPause();
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (mapLoadHelper != null) {
            mapLoadHelper.onDestroy();
            mapLoadHelper = null;
        }
        if (mapView != null) {
            mapView.onDestroy();
        }
    }

}
