package com.tofly.yxpc.ui.fragment;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.AdapterView;
import android.widget.CheckBox;
import android.widget.LinearLayout;

import androidx.annotation.Nullable;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.tofly.latte_core.base.LatteContract;
import com.tofly.latte_core.net.bean.BaseResponse;
import com.tofly.latte_core.utils.AccountManager;
import com.tofly.latte_core.utils.NetworkUtil;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.MyApp;
import com.tofly.yxpc.databinding.FragmentUncalibrateBinding;
import com.tofly.yxpc.entity.BaseUpdataEntity;
import com.tofly.yxpc.entity.DictEntity;
import com.tofly.yxpc.entity.FileBean;
import com.tofly.yxpc.entity.KbdDescribesBean;
import com.tofly.yxpc.entity.TwdhEntity;
import com.tofly.yxpc.entity.WaterMeterEntity;
import com.tofly.yxpc.entity.WaterMeterTaskEntity;
import com.tofly.yxpc.greenDao.DictEntityDao;
import com.tofly.yxpc.greenDao.TwdhEntityDao;
import com.tofly.yxpc.greenDao.WaterMeterTaskEntityDao;
import com.tofly.yxpc.ui.activity.ImageTextNavitActivity;
import com.tofly.yxpc.ui.adapter.BigPictureAdapter;
import com.tofly.yxpc.ui.impl.UpLoadImpl;
import com.tofly.yxpc.utils.DateUtils;
import com.tofly.yxpc.utils.RecyclerViewUtils;
import com.tofly.yxpc.utils.SpinnerUtils;
import com.tofly.yxpc.utils.db.DaoUtilsStore;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 76486
 */
public class UnCalibrateFragment extends PhotoFragment {

    private FragmentUncalibrateBinding binding;
    private BigPictureAdapter pictureAdapter;
    private final List<FileBean> photoList = new ArrayList<>();
    private WaterMeterEntity waterMeterEntity;
    private final List<DictEntity> dictEntities = new ArrayList<>();
    private boolean isFirstLoading = true;
    private Long navitId = null;

    @Override
    public void onTakePhotoSuccess(String pathFile) {
        FileBean fileBean = new FileBean();
        fileBean.setFilePath(pathFile);
        photoList.add(fileBean);
        pictureAdapter.setNewData(photoList);
        if (photoList.isEmpty()) {
            binding.rvListPhoto.setVisibility(View.GONE);
        } else {
            binding.rvListPhoto.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onTakeVideoSuccess(String videoFile) {

    }

    @Override
    public Object setLayout() {
        binding = FragmentUncalibrateBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected LatteContract.LattePresenter createPresenter() {
        return new UpLoadImpl();
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        setIsLocation(true);
        super.onCreate(savedInstanceState);
        Bundle bundle = this.getArguments();//得到从Activity传来的数据
        Long Id = null;
        if (bundle != null) {
            Id = bundle.getLong("Id");
        }
        waterMeterEntity = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryById(Id);
    }

    @Override
    public void onResume() {
        super.onResume();
        if (!isFirstLoading) {
            //如果不是第一次加载，刷新数据
            initRecyclerView();
        }
        isFirstLoading = false;
    }

    @Override
    public void initView() {
        super.initView();
        initRecyclerView();
        initListener();
    }

    private void initRecyclerView() {
        TwdhEntity twdhEntity = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilderOne(TwdhEntityDao.Properties.WaterNavigationInspects.like("%" + waterMeterEntity.getId() + "%"));
        initNavit(twdhEntity);
        if (waterMeterEntity.getWfbdFileList() != null) {
            photoList.addAll(waterMeterEntity.getWfbdFileList());
        }
        pictureAdapter = new BigPictureAdapter(photoList, true, size -> {

        });
        RecyclerViewUtils.initGrid(requireContext(), binding.rvListPhoto, 3, pictureAdapter);
        if (photoList.isEmpty()) {
            binding.rvListPhoto.setVisibility(View.GONE);
        } else {
            binding.rvListPhoto.setVisibility(View.VISIBLE);
        }
        if (waterMeterEntity.getWfDescribes() != null) {
            binding.edDescribe.setText(waterMeterEntity.getWfDescribes().getExceptionDescribe());
        }
        List<DictEntity> dictEntityList = DaoUtilsStore.getInstance().getDictEntityDbUtils().queryByQueryBuilder(DictEntityDao.Properties.DictCode.eq("enroll_info"));
        List<String> ides = null;
        if (waterMeterEntity.getWfDescribes() != null && waterMeterEntity.getWfDescribes().getExceptionType() != null) {
            ides = new ArrayList<>(Arrays.asList(waterMeterEntity.getWfDescribes().getExceptionType().split(",")));
        }
        for (DictEntity dictEntity : dictEntityList) {
            CheckBox checkBox = new CheckBox(requireContext());
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                checkBox.setTextColor(getContext().getColor(R.color.color_333));
                LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
                //设置边距
                layoutParams.setMargins(20, 12, 0, 0);
                //将以上的属性赋给LinearLayout
                checkBox.setLayoutParams(layoutParams);
                checkBox.setTextSize(16);
            }
            checkBox.setText(dictEntity.getItemText());
            checkBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
                if (isChecked) {
                    dictEntities.add(dictEntity);
                } else {
                    dictEntities.remove(dictEntity);
                }
            });
            if (ides != null && ides.contains(dictEntity.getId())) {
                checkBox.setChecked(true);
            }
            binding.llAbnormal.addView(checkBox, dictEntityList.indexOf(dictEntity));
        }
    }

    private void initListener() {
        binding.imgPhoto.setOnClickListener(v -> {
            if (photoList.size() < 9) {
                showPhoto(false);
            } else {
                onMessage("图片数量超过限制");
            }
        });
        binding.cbAbnormal.setOnClickListener(v -> binding.llAbnormal.setVisibility(binding.cbAbnormal.isChecked() ? View.VISIBLE : View.GONE));
        binding.cbNavit.setOnClickListener(v -> binding.llNavit.setVisibility(binding.cbNavit.isChecked() ? View.VISIBLE : View.GONE));
        binding.tvDraft.setOnClickListener(v -> {
            save();
            waterMeterEntity.setState(1);
            DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().insertOrReplace(waterMeterEntity);
            BaseUpdataEntity baseUpdata = DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().queryById(waterMeterEntity.getId());
            if (baseUpdata != null) {
                DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().delete(baseUpdata);
            }
            onMessage("保存成功");
            requireActivity().finish();
        });
        binding.tvSave.setOnClickListener(v -> {
            if (check()) {
                save();
                saveLocal("保存成功");
            }
        });
        binding.tvSubmit.setOnClickListener(v -> {
            if (check()) {
                save();
                if (NetworkUtil.isNetworkAvailable()) {
                    updateWaterMeter();
                } else {
                    saveLocal("网络异常，先保存本地");
                }
            }
        });
        binding.tvSee.setOnClickListener(v -> {
            Intent intent = new Intent(getContext(), ImageTextNavitActivity.class);
            intent.putExtra("id", String.valueOf(navitId));
            startActivity(intent);
        });
        binding.tvNew.setOnClickListener(v -> {
            if (MyApp.getInstance().isOverTime()) {
                Intent intent = new Intent(requireActivity(), ImageTextNavitActivity.class);
                intent.putExtra("waterMeterId", waterMeterEntity.getId());
                startActivityForResult(intent, 1102);
            }
        });
        if (waterMeterEntity.getState() == 3) {
            binding.layout.setVisibility(View.GONE);
        } else {
            binding.layout.setVisibility(View.VISIBLE);
        }
    }

    private void initNavit(TwdhEntity twdhEntity) {
        List<TwdhEntity> twdhEntityList = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryAll();
        List<String> stringList = new ArrayList<>();
        List<Long> longList = new ArrayList<>();
        stringList.add("请选择");
        for (TwdhEntity twdh : twdhEntityList) {
            stringList.add(twdh.getName() == null ? "" : twdh.getName());
            longList.add(twdh.getId());
        }
        SpinnerUtils.initSpinner(requireContext(), binding.spinnerNavit, stringList);
        binding.spinnerNavit.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                position--;
                if (position == -1) {
                    navitId = null;
                    binding.tvSee.setVisibility(View.GONE);
                } else {
                    navitId = twdhEntityList.get(position).getId();
                    binding.tvSee.setVisibility(View.VISIBLE);
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        if (!twdhEntityList.isEmpty() && twdhEntity != null) {
            int index = longList.indexOf(twdhEntity.getId()) + 1;
            if (index > 1) {
                binding.spinnerNavit.setSelection(index);
            }
        }
    }

    private void saveLocal(String message) {
        waterMeterEntity.setState(2);
        DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().insertOrReplace(waterMeterEntity);
        BaseUpdataEntity baseUpdataEntity = DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().queryById(waterMeterEntity.getId());
        if (baseUpdataEntity == null) {
            baseUpdataEntity = new BaseUpdataEntity();
        }
        baseUpdataEntity.setId(waterMeterEntity.getId());
        baseUpdataEntity.setCheckId(AccountManager.getUserId());
        baseUpdataEntity.setUserName(waterMeterEntity.getUserName());
        baseUpdataEntity.setType(2);
        if (!dictEntities.isEmpty()) {
            StringBuilder stringBuilder = new StringBuilder();
            for (DictEntity entity : dictEntities) {
                if (dictEntities.indexOf(entity) == 0) {
                    stringBuilder.append(entity.getItemText());
                } else {
                    stringBuilder.append(",").append(entity.getItemText());
                }
            }
            baseUpdataEntity.setExceptionType(stringBuilder.toString());
        }
        baseUpdataEntity.setWaterMeterNum(waterMeterEntity.getWaterMeterNum());
        baseUpdataEntity.setUserAdd(waterMeterEntity.getUserAddr());
        baseUpdataEntity.setSaveTime(DateUtils.getCurrentDate());
        DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().insertOrReplace(baseUpdataEntity);

        onMessage(message);
        requireActivity().finish();
    }

    private void updateWaterMeter() {
        updateTwdh();

        Map<String, Object> map = JSONObject.parseObject(JSON.toJSONString(waterMeterEntity), HashMap.class);
        if (waterMeterEntity.getBpFileList() != null && !waterMeterEntity.getBpFileList().isEmpty()) {
            List<File> files1 = new ArrayList<>();
            for (FileBean fileBean : waterMeterEntity.getBpFileList()) {
                files1.add(new File(fileBean.getFilePath()));
            }
            map.put("bpFileList", files1);
        }

        if (waterMeterEntity.getExceptionFileList() != null && !waterMeterEntity.getExceptionFileList().isEmpty()) {
            List<File> files2 = new ArrayList<>();
            if (waterMeterEntity.getExceptionFileList() != null) {
                for (FileBean fileBean : waterMeterEntity.getExceptionFileList()) {
                    files2.add(new File(fileBean.getFilePath()));
                }
            }
            map.put("exceptionFileList", files2);
        }

        if (waterMeterEntity.getWfbdFileList() != null && !waterMeterEntity.getWfbdFileList().isEmpty()) {
            List<File> files3 = new ArrayList<>();
            if (waterMeterEntity.getWfbdFileList() != null) {
                for (FileBean fileBean : waterMeterEntity.getWfbdFileList()) {
                    files3.add(new File(fileBean.getFilePath()));
                }
            }
            map.put("wfbdFileList", files3);
        }
        mPresenter.getResultThree(requireContext(), map);
    }

    /**
     * 上传本地绑定未上传的图文导航
     */
    private void updateTwdh() {
        if (!TextUtils.isEmpty(waterMeterEntity.getNavigationId())) {
            TwdhEntity twdhEntity = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryById(Long.parseLong(waterMeterEntity.getNavigationId()));
            if (twdhEntity != null && "0".equals(twdhEntity.getState())) {
                Map<String, Object> map = new HashMap<>();
                map.put("id", twdhEntity.getId());
                map.put("name", twdhEntity.getName());
                map.put("describe", twdhEntity.getDescribe());
                map.put("waterNavigationInspects", twdhEntity.getWaterNavigationInspects());
                if (twdhEntity.getFileList() != null && !twdhEntity.getFileList().isEmpty()) {
                    List<File> files = new ArrayList<>();
                    for (FileBean fileBean : twdhEntity.getFileList()) {
                        files.add(new File(fileBean.getFilePath()));
                    }
                    map.put("file", files);
                }
                List<File> dtFiles = new ArrayList<>();
                for (FileBean fileBean : twdhEntity.getMapFileList()) {
                    dtFiles.add(new File(fileBean.getFilePath()));
                }
                map.put("mapfilelist", dtFiles);
                mPresenter.getResultTwo(requireContext(), map);
            }
        }
    }

    private void save() {
        if (!TextUtils.isEmpty(binding.edDescribe.getText())) {
            if (waterMeterEntity.getWfDescribes() == null) {
                KbdDescribesBean wfDescribes = new KbdDescribesBean();
                waterMeterEntity.setWfDescribes(wfDescribes);
            }
            waterMeterEntity.getWfDescribes().setExceptionDescribe(binding.edDescribe.getText().toString());
        }

        waterMeterEntity.setWfbdFileList(photoList);
        waterMeterEntity.setNavigationId(navitId == null ? "" : String.valueOf(navitId));
        StringBuilder stringBuilder = new StringBuilder();
        for (DictEntity entity : dictEntities) {
            if (dictEntities.indexOf(entity) == 0) {
                stringBuilder.append(entity.getId());
            } else {
                stringBuilder.append(",").append(entity.getId());
            }
        }
        if (!dictEntities.isEmpty()) {
            if (waterMeterEntity.getWfDescribes() == null) {
                KbdDescribesBean wfDescribes = new KbdDescribesBean();
                waterMeterEntity.setWfDescribes(wfDescribes);
            }
            waterMeterEntity.getWfDescribes().setExceptionType(stringBuilder.toString());
        }
        waterMeterEntity.setExceptionFileList(null);
        waterMeterEntity.setKbdDescribes(null);
    }

    private boolean check() {
        if (photoList.isEmpty()) {
            onMessage("请添加异常照片");
            return false;
        }
        return true;
    }

    @Override
    public <T> void getResultOne(T response) {

    }

    @Override
    public <T> void getResultTwo(T response) {
        BaseResponse baseResponse = (BaseResponse) response;
        if (baseResponse != null && "200".equals(baseResponse.getCode())) {
            TwdhEntity twdhEntity = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryById(Long.parseLong(waterMeterEntity.getNavigationId()));
            if (twdhEntity != null) {
                twdhEntity.setState(null);
                new Thread(() -> {
                    String[] id = twdhEntity.getWaterNavigationInspects().split(",");
                    for (String str : id) {
                        if (!TextUtils.isEmpty(str)) {
                            List<TwdhEntity> twdhEntityList = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilder(TwdhEntityDao.Properties.WaterNavigationInspects.like("%" + str + "%"));
                            for (TwdhEntity entity1 : twdhEntityList) {
                                String waterNavigationInspects = entity1.getWaterNavigationInspects().replace("," + str, "").replace(str, "");
                                entity1.setWaterNavigationInspects(waterNavigationInspects);
                                if (TextUtils.isEmpty(entity1.getWaterNavigationInspects())) {
                                    DaoUtilsStore.getInstance().getTwdhEntityDbUtils().delete(entity1);
                                } else {
                                    DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertOrReplace(entity1);
                                }
                            }
                        }
                    }
                    DaoUtilsStore.getInstance().getTwdhEntityDbUtils().insertOrReplace(twdhEntity);
                }).start();
            }
        }
    }

    @Override
    public <T> void getResultThree(T response) {
        BaseResponse baseResponse = (BaseResponse) response;
        if (baseResponse != null && "200".equals(baseResponse.getCode())) {
            onMessage("数据上传完毕");
            waterMeterEntity.setState(3);
            DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().insertOrReplace(waterMeterEntity);
            BaseUpdataEntity baseUpdata = DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().queryById(waterMeterEntity.getId());
            if (baseUpdata != null) {
                DaoUtilsStore.getInstance().getBaseUpdataEntityDbUtils().delete(baseUpdata);
            }
            List<WaterMeterTaskEntity> waterMeterTaskEntities = DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().queryByQueryBuilder(WaterMeterTaskEntityDao.Properties.CheckUser.eq(AccountManager.getUserId()));
            for (WaterMeterTaskEntity entity : waterMeterTaskEntities) {
                boolean flag = true;
                for (WaterMeterEntity waterMeter : entity.getYxswWaterMeterInspectDto()) {
                    WaterMeterEntity dbWaterMeter = DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().queryById(waterMeter.getId());
                    waterMeter.setState(dbWaterMeter.getState());
                    if (3 != dbWaterMeter.getState()) {
                        flag = false;
                    }
                }
                if (flag) {
                    entity.setState("2");
                }
            }
            DaoUtilsStore.getInstance().getWaterMeterTaskEntityDbUtils().insertMulti(waterMeterTaskEntities, false);
            requireActivity().finish();
        } else {
            saveLocal("上传失败，先保存本地");
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 1102 && resultCode == 0) {
            TwdhEntity twdhEntity = DaoUtilsStore.getInstance().getTwdhEntityDbUtils().queryByQueryBuilderOne(TwdhEntityDao.Properties.WaterNavigationInspects.like("%" + waterMeterEntity.getId() + "%"));
            navitId = twdhEntity.getId();
            waterMeterEntity.setNavigationId(String.valueOf(navitId));
            DaoUtilsStore.getInstance().getWaterMeterEntityDbUtils().insertOrReplace(waterMeterEntity);
            initNavit(twdhEntity);
        }
    }
}
