package com.tofly.yxpc.ui.impl;

import android.content.Context;

import com.alibaba.fastjson.JSON;
import com.tofly.latte_core.base.LattePresenterImpl;
import com.tofly.latte_core.config.Constants;
import com.tofly.latte_core.net.LatteNetWorkApi;
import com.tofly.latte_core.net.bean.BaseResponse;
import com.tofly.latte_core.net.helper.RxHelper;
import com.tofly.latte_core.net.observer.BaseObserver;
import com.tofly.latte_core.net.observer.MyObserver;
import com.tofly.yxpc.base.Api;
import com.tofly.yxpc.entity.user.LoginBean;
import com.tofly.yxpc.entity.user.UserInfo;
import com.tofly.yxpc.utils.RequestBodyUtils;

import java.util.Map;
import java.util.Objects;

/**
 * @author ychk
 */
public class LoginImpl extends LattePresenterImpl {

    @Override
    public void getResultOne(Context context, Map<String, Object> map) {
        super.getResultOne(context, map);
        LatteNetWorkApi.getApiService(Api.class)
                .login(RequestBodyUtils.toRequestBodyJson(JSON.toJSONString(map)))
                .compose(RxHelper.observableIO2Main(context))
                .subscribe(new MyObserver<LoginBean>(context) {
                    @Override
                    public void onSuccess(LoginBean demo) {
                        if (isViewAttached()) {
                            getView().getResultOne(demo);
                        }
                    }

                    @Override
                    public void onFailure(Throwable e, String errorMsg) {
                        if (isViewAttached()) {
                            getView().onMessage(errorMsg);
                        }
                    }
                });
    }

    @Override
    public void getResultTwo(Context context, Map<String, Object> map) {
        super.getResultTwo(context, map);
        LatteNetWorkApi.getApiService(Api.class)
                .getUserInfo()
                .compose(RxHelper.observableIO2Main(context))
                .subscribe(new BaseObserver<UserInfo>() {
                    @Override
                    public void onSuccess(UserInfo demo) {
                        if (isViewAttached()) {
                            getView().getResultTwo(demo);
                        }
                    }

                    @Override
                    public void onFailure(Throwable e, String errorMsg) {
                        if (isViewAttached()) {
                            getView().onMessage(errorMsg);
                        }
                    }
                });
    }

    @Override
    public void getResultThree(Context context, Map<String, Object> map) {
        super.getResultThree(context, map);
        LatteNetWorkApi.getApiService(Api.class)
                .resetPwd(RequestBodyUtils.toRequestBodyJson(JSON.toJSONString(map)))
                .compose(RxHelper.observableIO2Main(context))
                .subscribe(new MyObserver<BaseResponse>(context) {
                    @Override
                    public void onSuccess(BaseResponse demo) {
                        if (isViewAttached()) {
                            if (Objects.equals(demo.getCode(), Constants.SUCCESS)) {
                                getView().getResultThree(demo.getCode());
                            } else {
                                getView().onMessage(demo.getMessage());
                                getView().getResultThree(null);
                            }
                        }
                    }

                    @Override
                    public void onFailure(Throwable e, String errorMsg) {
                        if (isViewAttached()) {
                            getView().onMessage(errorMsg);
                            getView().getResultThree(null);
                        }
                    }
                });
    }

}
