package com.tofly.yxpc.utils;

import android.content.Context;

import java.io.File;
import java.io.IOException;

import id.zelory.compressor.Compressor;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;

/**
 * @author ychk
 * form数据和文件类型转化
 */
public class RequestBodyUtils {

    /**
     * 文字信息转化对象
     */
    public static RequestBody toRequestBodyString(String value) {
        MediaType mediaType = MediaType.parse("text/plain;charset=UTF-8");
        return RequestBody.create(mediaType, value);
    }

    /**
     * 文件转化对象
     */
    public static RequestBody toRequestBodyFile(File value, Context context) {
        RequestBody body = null;
        try {
            //首先判断是不是图片文件
            String name = value.getName();
            MediaType mediaType = MediaType.parse("multipart/form-data");
            if (name.endsWith(".png") || name.endsWith(".jpg") || name.endsWith(".jpeg")) {
                double fileOrFilesSize = FileSizeUtils.getFileOrFilesSize(value.getPath(), 3);
                //在判断是否需要压缩1M以上就默认压缩
                if (fileOrFilesSize >= 1) {
                    File file = new Compressor(context)
                            .compressToFile(value);
                    body = RequestBody.create(mediaType, file);
                } else {
                    body = RequestBody.create(mediaType, value);
                }
            } else {
                body = RequestBody.create(mediaType, value);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return body;
    }

    /**
     * json转化对象
     */
    public static RequestBody toRequestBodyJson(String value) {
        MediaType mediaType = MediaType.parse("application/json");
        return RequestBody.create(mediaType, value);
    }
public static MultipartBody.Part toMultiPartBodyFile(String key, File value, Context context) {
        MultipartBody.Part part = null;
        RequestBody body = null;
            //首先判断是不是图片文件
            String name = value.getName();
            MediaType mediaType = MediaType.parse("image/*");

            if (name.endsWith(".png") || name.endsWith(".jpg") || name.endsWith(".jpeg")) {
                body = RequestBody.create(mediaType, value);
            } else {
                body = RequestBody.create(mediaType, value);
            }

        if (body != null) {
            part = MultipartBody.Part.createFormData(key, value.getName(), body);
        }
        return part;
    }
}
