package com.tofly.yxpc.utils;

import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;

import androidx.core.content.FileProvider;

import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.base.NetWorkUrl;

import java.io.File;
import java.util.List;
import java.util.Objects;

/**
 * @author ychk
 */
public class WpsUtils {
    public WpsInterface wpsInterface;
    private WpsCloseListener wpsCloseListener = null;
    private Boolean canWrite;
    private Activity mActivity;

    public WpsUtils(WpsInterface wpsInterface, File file, Boolean canWrite, Activity activity) {
        this.wpsInterface = wpsInterface;
        this.canWrite = canWrite;
        this.mActivity = activity;
        isWps(file);
    }

    /**
     * 是否按装了wps
     */
    private void isWps(File file) {
        if (isAvilible(mActivity)) {
            openDocument(file);
        } else {
            try {
                // 从市场上下载
                Uri uri = Uri.parse("market://details?id=cn.wps.moffice_eng");
                Intent it = new Intent(Intent.ACTION_VIEW, uri);
                mActivity.startActivity(it);
            } catch (Exception ignore) {
                ToastUtils.showToast("请下载wps");
            }

        }
    }

    /**
     * 打开入口
     */
    public void openDocument(File file) {
        try {
            wpsCloseListener = new WpsCloseListener();
            IntentFilter filter = new IntentFilter();
            //按下返回键
            filter.addAction("com.kingsoft.writer.back.key.down");
            //按下home键
            filter.addAction("com.kingsoft.writer.home.key.down");
            //保存
            filter.addAction("cn.wps.moffice.file.save");
            //关闭
            filter.addAction("cn.wps.moffice.file.close");
            //注册广播
            mActivity.registerReceiver(wpsCloseListener, filter);
            if (file.exists()) {
                openDocWithSimple(file);
            } else {
                openDocFile(NetWorkUrl.getFileUrl(file.getPath()));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 打开本地文件
     */
    public void openDocWithSimple(File file) {
        try {
            Intent intent = mActivity.getPackageManager().getLaunchIntentForPackage("cn.wps.moffice_eng");
            Bundle bundle = new Bundle();
            //打开模式
            bundle.putString(Define.OPEN_MODE, Define.NORMAL);
            //以修订模式打开
            bundle.putBoolean(Define.ENTER_REVISE_MODE, true);
            bundle.putBoolean(Define.SEND_SAVE_BROAD, true);
            bundle.putBoolean(Define.SEND_CLOSE_BROAD, true);
            bundle.putBoolean(Define.HOME_KEY_DOWN, true);
            bundle.putBoolean(Define.BACK_KEY_DOWN, true);
            bundle.putBoolean(Define.ENTER_REVISE_MODE, true);
            bundle.putBoolean(Define.IS_SHOW_VIEW, false);
            bundle.putBoolean(Define.AUTO_JUMP, true);
            //设置广播
            bundle.putString(Define.THIRD_PACKAGE, mActivity.getPackageName());
            assert intent != null;
            intent.setAction(Intent.ACTION_VIEW);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                Uri contentUri = FileProvider.getUriForFile(mActivity,
                        mActivity.getPackageName() + ".fileprovider", file);
                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                intent.setDataAndType(contentUri, "*/*");
            } else {
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.setDataAndType(Uri.fromFile(file), "*/*");
            }
            intent.putExtras(bundle);
            mActivity.startActivity(intent);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 打开文档
     */
    private void openDocFile(String path) {
        try {
            Intent intent = mActivity.getPackageManager().getLaunchIntentForPackage("cn.wps.moffice_eng");
            Bundle bundle = new Bundle();
            if (canWrite) {
                bundle.putString(Define.OPEN_MODE, Define.NORMAL);
                //以修订模式打开
                bundle.putBoolean(Define.ENTER_REVISE_MODE, true);
            } else {
                bundle.putString(Define.OPEN_MODE, Define.READ_ONLY);
            }
            //打开模式
            bundle.putBoolean(Define.SEND_SAVE_BROAD, true);
            bundle.putBoolean(Define.SEND_CLOSE_BROAD, true);
            bundle.putBoolean(Define.HOME_KEY_DOWN, true);
            bundle.putBoolean(Define.BACK_KEY_DOWN, true);
            bundle.putBoolean(Define.ENTER_REVISE_MODE, true);
            bundle.putBoolean(Define.IS_SHOW_VIEW, false);
            bundle.putBoolean(Define.AUTO_JUMP, true);
            //设置广播
            bundle.putString(Define.THIRD_PACKAGE, mActivity.getPackageName());
            assert intent != null;
            intent.setAction(Intent.ACTION_VIEW);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.setData(Uri.parse(path));
            intent.putExtras(bundle);
            mActivity.startActivity(intent);
        } catch (ActivityNotFoundException e) {
            e.printStackTrace();
        }
    }

    public interface WpsInterface {
        void doRequest(String filePath);

        void doFinish();
    }

    /**
     * 广播接收器
     */
    private class WpsCloseListener extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            try {
                if (Objects.equals(intent.getAction(), "cn.wps.moffice.file.save")) {
                    String fileSavePath = Objects.requireNonNull(intent.getExtras()).getString(Define.SAVE_PATH);
                    if (canWrite) {
                        wpsInterface.doRequest(fileSavePath);
                    }
                } else if (Objects.equals(intent.getAction(), "cn.wps.moffice.file.close") ||
                        "com.kingsoft.writer.back.key.down".equals(intent.getAction())) {
                    wpsInterface.doFinish();
                    mActivity.unregisterReceiver(wpsCloseListener);//注销广播
                }
            } catch (Exception e) {
                Log.d("wps", Objects.requireNonNull(e.getMessage()));
            }

        }
    }

    public void appBack() {
//        //获取ActivityManager
//        ActivityManager mAm = (ActivityManager) mActivity.getSystemService(Context.ACTIVITY_SERVICE);
//        //获得当前运行的task
//        List<ActivityManager.RunningTaskInfo> taskList = mAm.getRunningTasks(100);
//        //找到当前应用的task，并启动task的栈顶activity，达到程序切换到前台
//        for (ActivityManager.RunningTaskInfo rti : taskList) {
//            if (rti.topActivity.getPackageName().equals(mActivity.getPackageName())) {
//                Intent LaunchIntent = new Intent(Intent.ACTION_MAIN);
//                ComponentName cn = new ComponentName(mActivity.getPackageName(), rti.topActivity.getClassName());
//                LaunchIntent.setComponent(cn);
//                mActivity.startActivity(LaunchIntent);
//                break;
//            }
//        }

    }

    /**
     * 验证手机是否安装了wps
     */
    private static boolean isAvilible(Context context) {
        final PackageManager packageManager = context.getPackageManager();
        List<PackageInfo> pInfo = packageManager.getInstalledPackages(0);
        int length = pInfo.size();
        for (int i = 0; i < length; i++) {
            if ("cn.wps.moffice_eng".equalsIgnoreCase(pInfo.get(i).packageName)) {
                return true;
            }
        }
        return false;
    }
}
