package com.tofly.yxpc.utils.update;

import android.app.DownloadManager;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;

import com.tofly.latte_core.utils.ActivityUtils;
import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.R;
import com.tofly.yxpc.base.NetWorkUrl;
import com.tofly.yxpc.ui.activity.LoginActivity;

import java.io.File;
import java.text.MessageFormat;
import java.util.Timer;
import java.util.TimerTask;

/**
 * @author ychk
 */
public class UpdateManager {
    private ProgressBar mProgressBar;//进度条
    private TextView tv1;
    private TextView tv2;
    private AlertDialog mDownloadDialog;//对话框

    private String mSavePath;//apk保存地址
    private int mProgress;//进度值

    private boolean mIsCancel = false;//是否取消下载标示符

    private static final int DOWNLOADING = 1;//apk下载中
    private static final int DOWNLOAD_FINISH = 2;//apk下载完毕
    private static final int DOWNLOAD_CANCEL = 3;
    private static final int DOWNLOAD_FAILED = 4;
    private long length;

    private static final int HAN_CHECK = 4;
    private static final int HAN_TOAST = 5;

    private Context mContext;//上下文

    private ProgressDialog checkProgressDia;

    private VersionRepository versionRepository;

    private VersionInfo.VersionBean mLastVersionInfo = null;

    private boolean isDownload = false;

    private VersionUpdateHintDialog updateHintDialog;

    public UpdateManager(Context context) {
        mContext = context;
        versionRepository = VersionRepository.getInstance();
    }

    @SuppressWarnings("HandlerLeak")
    private Handler mUpdateProgressHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case DOWNLOADING:
                    // 设置进度条
                    long downloaded = (long) msg.arg1;
                    long total = (long) msg.arg2;
                    if (total >= 0 && downloaded >= 0) {
                        mProgress = Math.round(downloaded * 100 / total);
                        mProgressBar.setIndeterminate(false);
                        mProgressBar.setProgress(mProgress);
                        tv1.setText(MessageFormat.format("{0}/100", mProgress));
                    } else {
                        mProgressBar.setIndeterminate(true);
                    }

                    if (downloaded >= 0) {
                        length = downloaded;
                        double len = (double) length / 1024.00 / 1024;
                        tv2.setText(String.format("%.2fM", len));
                    }
                    break;
                case DOWNLOAD_CANCEL:
                    isDownload = false;
                    mDownloadDialog.dismiss();
                    ToastUtils.showToast("用户取消下载");
                    break;
                case DOWNLOAD_FINISH:
                    // 隐藏当前下载对话框
                    isDownload = false;
                    mDownloadDialog.dismiss();
                    // 安装 APK 文件
                    checkAppCanInstall();
                    break;
                case DOWNLOAD_FAILED:
                    isDownload = false;
                    tv1.setText("下载失败");
                    mDownloadDialog.dismiss();
                    ToastUtils.showToast("下载失败!");
                    break;
                default:
                    break;
            }
        }

        ;
    };

    /**
     * 检测软件是否需要更新
     */
    public void checkUpdate(boolean isShow) {
        if (isDownload) {
            return;
        }
        isDownload = true;
        checkProgressDia = new ProgressDialog(this.mContext);
        checkProgressDia.setMessage("正在检测新版本");
        checkProgressDia.setTitle("提 示");
        checkProgressDia.setCanceledOnTouchOutside(false);
        checkProgressDia.setCancelable(true);
        checkProgressDia.show();
        versionRepository.getLastVersionInfo(mContext, new VersionCallback() {
            @Override
            public void onGetLastVersionInfo(VersionInfo.VersionBean versionInfo) {
                if (versionInfo != null) {
                    mLastVersionInfo = versionInfo;
                    if (isUpdate(mLastVersionInfo)) {
                        showNoticeDialog(mLastVersionInfo, false);
                    } else {
                        if (isShow) {
                            ToastUtils.showToast("已是最新版本");
                        }
                    }
                } else {
                    if (isShow) {
                        ToastUtils.showToast("已是最新版本");
                    }
                }
                isDownload = false;
                checkProgressDia.dismiss();
            }

            @Override
            public void onError(String errMsg) {
                isDownload = false;
                checkProgressDia.dismiss();
                ToastUtils.showToast(errMsg);
            }
        });
    }

    public void getVersionInfo(VersionSource.IVersionCallback versionCallback) {
        versionRepository.getLastVersionInfo(mContext, versionCallback);
    }

    /**
     * 与本地版本比较判断是否需要更新
     */
    protected boolean isUpdate(VersionInfo.VersionBean versionInfo) {
        int serverVersion = 1;
        int localVersion = 1;
        try {
            String version_code = versionInfo.getVersionNumber().replace(".", "");
            serverVersion = Integer.parseInt(version_code);
        } catch (NumberFormatException e) {
            Log.e(this.getClass().getSimpleName(), e.toString());
        }
        //获得当前apk版本号versionCode
        localVersion = AppUtil.getLocalVersionCode(mContext);
        return serverVersion > localVersion;
    }

    /**
     * 有更新时显示提示对话框
     */
    protected void showNoticeDialog(VersionInfo.VersionBean newVersion, boolean isMustUpdate) {
        if (updateHintDialog == null) {
            updateHintDialog = new VersionUpdateHintDialog(mContext, newVersion, view -> {
                // 下载文件
                downloadAPK();
                // 显示下载对话框
                showDownloadDialog(isMustUpdate);
            }, view -> {
                if (isMustUpdate) {
                    gotoUiLogin();
                }
            });
        }
        updateHintDialog.setCanceledOnTouchOutside(!isMustUpdate);
        updateHintDialog.setCancelable(!isMustUpdate);
        updateHintDialog.show();
    }

    private void gotoUiLogin() {
        ActivityUtils.getInstance().finishAllActivity();
        Intent intent = new Intent();
        intent.setClass(mContext, LoginActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        mContext.startActivity(intent);
    }

    /**
     * 显示正在下载对话框
     */
    protected void showDownloadDialog(boolean isMustUpdate) {
        AlertDialog.Builder builder = new AlertDialog.Builder(mContext);
        View view = LayoutInflater.from(mContext).inflate(R.layout.dialog_progress, null);
        TextView titleTv = (TextView) view.findViewById(R.id.tv_dialog_title);
        titleTv.setText("下载中");
        mProgressBar = (ProgressBar) view.findViewById(R.id.myprogress);
        tv1 = (TextView) view.findViewById(R.id.progressbar_tv1);
        tv2 = (TextView) view.findViewById(R.id.progressbar_tv2);
        builder.setView(view);

        // 设置按钮(左)
        if (!isMustUpdate) {
             builder.setPositiveButton("取消", (dialog, which) -> {
                // 隐藏当前对话框
                dialog.dismiss();
                // 设置下载状态为取消
                mIsCancel = true;
            });

            // 设置按钮(右)
            //  builder.setNegativeButton("隐藏", (dialog, which) -> dialog.dismiss());
        }

        mDownloadDialog = builder.create();
        mDownloadDialog.setCancelable(false);
        mDownloadDialog.setCanceledOnTouchOutside(false);
        mDownloadDialog.show();
    }

    /**
     * 开启新线程下载文件
     */
    private void downloadAPK() {
        if (mLastVersionInfo == null || mLastVersionInfo.getFileInfos() == null || mLastVersionInfo.getFileInfos().size() == 0) {
            return;
        }

        String versionName = mLastVersionInfo.getFileInfos().get(0).getFileName();
        String versionPath = mLastVersionInfo.getFileInfos().get(0).getFilePath();
        isDownload = true;
        String url = NetWorkUrl.getFileUrl(versionPath);
        Log.w(getClass().getName(), "downloadAPK: url = " + url);
        Uri uri = Uri.parse(url);
        final DownloadManager.Request request = new DownloadManager.Request(uri);
        request.setNotificationVisibility(DownloadManager.Request
                .VISIBILITY_VISIBLE_NOTIFY_COMPLETED);
        mSavePath = "apk/" + versionName;
        request.setDestinationInExternalFilesDir(mContext, Environment.DIRECTORY_DOWNLOADS, mSavePath);
        request.setTitle(mContext.getResources().getString(R.string.app_name) + "更新中...");
        request.setDescription("下载完后请点击打开");
        request.setMimeType("application/vnd.android.package-archive");


        final Timer myTimer = new Timer();
        final DownloadManager dm = (DownloadManager) mContext.getSystemService(Context.DOWNLOAD_SERVICE);
        final long downloadId = dm.enqueue(request);
        final DownloadManager.Query query = new DownloadManager.Query().setFilterById(downloadId);
        myTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                if (!mIsCancel) {
                    Cursor cursor = dm.query(query);
                    if (cursor != null) {
                        if (cursor.moveToFirst()) {
                            int status = cursor.getInt(cursor.getColumnIndex(DownloadManager.COLUMN_STATUS));
                            if (status == DownloadManager.STATUS_SUCCESSFUL) {
                                myTimer.cancel();
                                mUpdateProgressHandler.sendEmptyMessage(DOWNLOAD_FINISH);
                            } else if (status == DownloadManager.STATUS_FAILED) {
                                dm.remove(downloadId);
                                myTimer.cancel();
                                mUpdateProgressHandler.sendEmptyMessage(DOWNLOAD_FAILED);
                            } else {
                                int bytesDownloaded = cursor.getInt(cursor.getColumnIndex(DownloadManager.COLUMN_BYTES_DOWNLOADED_SO_FAR));
                                int bytesTotal = cursor.getInt(cursor.getColumnIndex(DownloadManager.COLUMN_TOTAL_SIZE_BYTES));
                                cursor.close();
                                final int dlProgress = (bytesDownloaded * 100 / bytesTotal);
                                if (dlProgress == 100) {
                                    myTimer.cancel();
                                    mUpdateProgressHandler.sendEmptyMessage(DOWNLOAD_FINISH);
                                } else {
                                    mUpdateProgressHandler.sendMessage(
                                            mUpdateProgressHandler.obtainMessage(DOWNLOADING, bytesDownloaded, bytesTotal));
                                }
                            }
                        }
                    }
                } else {
                    dm.remove(downloadId);
                    myTimer.cancel();
                    mUpdateProgressHandler.sendEmptyMessage(DOWNLOAD_CANCEL);
                }
            }
        }, 0, 10);
    }

    private void checkAppCanInstall() {
        boolean installAllowed;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            installAllowed = mContext.getPackageManager().canRequestPackageInstalls();
            if (!installAllowed) {
                Intent intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, Uri.parse("package:" + mContext.getPackageName()));
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                mContext.startActivity(intent);
            }
        }
        installAPK();
    }

    /**
     * 下载到本地后执行安装
     */
    protected void installAPK() {
        File rootFile = mContext.getExternalFilesDir(Environment.DIRECTORY_DOWNLOADS);
        File apkFile = new File(rootFile, mSavePath);
        if (!apkFile.exists()) {
            ToastUtils.showToast("安装包未找到，退出安装!");
            return;
        }

        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            //参数1 上下文, 参数2 Provider主机地址和清单文件中保持一致   参数3 共享的文件
            Uri apkUri = FileUtil.getUri(mContext, apkFile);
            //添加这一句表示对目标应用临时授权该Uri所代表的文件
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            intent.setDataAndType(apkUri, "application/vnd.android.package-archive");
        } else {
            intent.setDataAndType(Uri.fromFile(apkFile), "application/vnd.android.package-archive");
        }
        mContext.startActivity(intent);
    }
}
