package com.tofly.yxpc.utils.update;

import android.content.Context;

import com.tofly.latte_core.config.Constants;
import com.tofly.latte_core.net.LatteNetWorkApi;
import com.tofly.latte_core.net.helper.RxHelper;
import com.tofly.latte_core.net.observer.BaseObserver;
import com.tofly.yxpc.base.Api;
import com.tofly.yxpc.utils.RequestBodyUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * @author ychk
 */
public class VersionRemoteSource implements VersionSource {

    private static VersionRemoteSource INSTANCE;

    public static synchronized VersionRemoteSource getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new VersionRemoteSource();
        }
        return INSTANCE;
    }

    private VersionRemoteSource() {

    }

    /**
     * 获取最新版本app的版本信息
     */
    @Override
    public void getLastVersionInfo(Context context, IVersionCallback versionCallback) {
        Map<String, Object> map = new HashMap<>(5);
//        map.put("userId", AccountManager.getUserId());
//        map.put("nowVersion", BuildConfig.VERSION_NAME);
        LatteNetWorkApi.getApiService(Api.class)
                .getAppVersion(RequestBodyUtils.toRequestBodyJson("{}"))
                .compose(RxHelper.observableIO2Main(context))
                .subscribe(new BaseObserver<VersionInfo>() {
                    @Override
                    public void onSuccess(VersionInfo versionInfo) {
                        if (versionInfo.getCode() == null || Constants.SUCCESS.equals(versionInfo.getCode())) {
                            if (versionInfo.getRecords() != null && !versionInfo.getRecords().isEmpty() && versionInfo.getRecords().get(0) != null) {
                                versionCallback.onGetLastVersionInfo(versionInfo.getRecords().get(0));
                            } else {
                                versionCallback.onGetLastVersionInfo(null);
                            }
                        } else {
                            versionCallback.onError(versionInfo.getMessage());
                        }
                    }

                    @Override
                    public void onFailure(Throwable e, String errorMsg) {
                        versionCallback.onError(errorMsg);
                    }
                });
    }
}
