package com.tofly.yxpc.widget;

import android.app.Dialog;
import android.content.Context;
import android.text.method.PasswordTransformationMethod;
import android.text.method.TransformationMethod;
import android.view.View;
import android.widget.EditText;

import androidx.annotation.NonNull;

import com.tofly.latte_core.utils.AccountManager;
import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.R;

import java.util.regex.Pattern;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

/**
 * @author ychk
 */
public class PasswordDialog extends Dialog {

    @BindView(R.id.et_old_pass)
    EditText etOldPass;
    @BindView(R.id.et_new_pass)
    EditText etNewPass;
    @BindView(R.id.et_confirm_pass)
    EditText etConfirmPass;

    private String oldPass;
    private String newPass;
    private final OnDialogSureListener onDialogSureListener;

    public PasswordDialog(@NonNull Context context, boolean isFirst, OnDialogSureListener onDialogSureListener) {
        super(context);
        setContentView(R.layout.dialog_password);
        ButterKnife.bind(this);
        this.setCancelable(false);
        this.onDialogSureListener = onDialogSureListener;
        if (isFirst) {
            etOldPass.setText(AccountManager.getUserPassword());
            etOldPass.setVisibility(View.GONE);
        }
        this.show();
    }

    private boolean checkUserInfo() {
        oldPass = etOldPass.getText().toString();
        newPass = etNewPass.getText().toString();
        String confirmPass = etConfirmPass.getText().toString();
        boolean isPass = false;
        if (oldPass.isEmpty()) {
            ToastUtils.showToast("旧密码为空");
            etOldPass.requestFocus();
        } else if (newPass.isEmpty()) {
            ToastUtils.showToast("新密码为空");
            etNewPass.requestFocus();
        } else if (validFail(newPass)) {
            ToastUtils.showToast("新密码格式错误");
            etNewPass.requestFocus();
        } else if (confirmPass.isEmpty()) {
            ToastUtils.showToast("确认密码为空");
            etConfirmPass.requestFocus();
        } else if (validFail(confirmPass)) {
            ToastUtils.showToast("确认密码格式错误");
            etConfirmPass.requestFocus();
        } else if (!newPass.equals(confirmPass)) {
            ToastUtils.showToast("新密码和确认密码不一致");
            etConfirmPass.requestFocus();
        } else {
            isPass = true;
        }
        return isPass;
    }

    public static boolean validFail(String password) {
        //大小写字母 数字 特殊字符 至少包含2种
        //  String x = "^(?![A-Z]*$)(?![a-z]*$)(?![0-9]*$)(?![^a-zA-Z0-9]*$)\\S+$";
        //大小写字母 数字 特殊字符 至少包含三种
        String x = "^(?![a-zA-Z]+$)(?![A-Z0-9]+$)(?![A-Z\\W_]+$)(?![a-z0-9]+$)(?![a-z\\W_]+$)(?![0-9\\W_]+$)[a-zA-Z0-9\\W_]{8,16}$";
        return !Pattern.matches(x, password);
    }

    @OnClick({R.id.img_eye_old, R.id.img_eye_new, R.id.img_eye_confirm, R.id.btn_cancel, R.id.btn_sure})
    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.img_eye_old:
                TransformationMethod transformationMethod = etOldPass.getTransformationMethod();
                if (transformationMethod instanceof PasswordTransformationMethod) {
                    etOldPass.setTransformationMethod(null);
                } else {
                    etOldPass.setTransformationMethod(PasswordTransformationMethod.getInstance());
                }
                break;
            case R.id.img_eye_new:
                TransformationMethod transformationMethodNew = etNewPass.getTransformationMethod();
                if (transformationMethodNew instanceof PasswordTransformationMethod) {
                    etNewPass.setTransformationMethod(null);
                } else {
                    etNewPass.setTransformationMethod(PasswordTransformationMethod.getInstance());
                }
                break;
            case R.id.img_eye_confirm:
                TransformationMethod transformationMethodConfirm = etConfirmPass.getTransformationMethod();
                if (transformationMethodConfirm instanceof PasswordTransformationMethod) {
                    etConfirmPass.setTransformationMethod(null);
                } else {
                    etConfirmPass.setTransformationMethod(PasswordTransformationMethod.getInstance());
                }
                break;
            case R.id.btn_cancel:
                dismiss();
                break;
            case R.id.btn_sure:
                if (checkUserInfo()) {
                    if (onDialogSureListener != null) {
                        onDialogSureListener.onPutIsPass(oldPass, newPass);
                        dismiss();
                    }
                }
                break;
            default:
                break;
        }
    }

    public interface OnDialogSureListener {
        void onPutIsPass(String oldPass, String newPass);
    }
}
