package com.tofly.yxpc.widget;

import android.app.Activity;
import android.content.Context;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.AttributeSet;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.coordinatorlayout.widget.CoordinatorLayout;

import com.amap.api.services.help.Inputtips;
import com.amap.api.services.help.InputtipsQuery;
import com.amap.api.services.help.Tip;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.tofly.latte_core.base.BaseActivity;
import com.tofly.latte_core.utils.ScreenUtils;
import com.tofly.yxpc.R;
import com.tofly.yxpc.utils.SoftInputUtils;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

/**
 * @author ychk
 */
public class SearchMapSheet extends LinearLayout implements TextWatcher, Inputtips.InputtipsListener {

    @BindView(R.id.iv_back)
    ImageView ivBack;
    @BindView(R.id.et_search_map)
    EditText etSearchMap;
    @BindView(R.id.ll_delete_map)
    LinearLayout llDeleteMap;

    private Context context;
    private int bottomSheetTop = 0;
    private BottomSheetBehavior<SearchMapSheet> bottomSheetBehavior = null;
    private textInputListener textInputListener = null;

    public SearchMapSheet(Context context) {
        this(context, null);
    }

    public SearchMapSheet(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public SearchMapSheet(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.context = context;
        initView();
    }

    private void initView() {
        inflate(context, R.layout.sheet_search_map, this);
        ButterKnife.bind(this);
        bottomSheetTop = BaseActivity.getViewHeight(context) - ScreenUtils.dip2px(context, 20);
        etSearchMap.addTextChangedListener(this);
    }

    public void initDetail(onBottomListener listener, textInputListener textInputListener) {
        this.textInputListener = textInputListener;
        bottomSheetBehavior = BottomSheetBehavior.from(this);
        bottomSheetBehavior.setPeekHeight(0);
        bottomSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);
        bottomSheetBehavior.setHideable(true);
        CoordinatorLayout.LayoutParams layoutParams = (CoordinatorLayout.LayoutParams) this.getLayoutParams();
        layoutParams.height = bottomSheetTop;
        setLayoutParams(layoutParams);
        float[] offset = {0};
        bottomSheetBehavior.setBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View view, int i) {
                CoordinatorLayout.LayoutParams params = (CoordinatorLayout.LayoutParams) view.getLayoutParams();
                switch (i) {
                    case BottomSheetBehavior.STATE_DRAGGING:
                        params.height = bottomSheetTop;
                        view.setLayoutParams(params);
                        break;
                    case BottomSheetBehavior.STATE_SETTLING:
                        if (offset[0] >= 0 && offset[0] <= 1) {
                            SoftInputUtils.hideSoft((Activity) context);
                            etSearchMap.clearFocus();
                            if (offset[0] > 0.9 && offset[0] < 1) {
                                bottomSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);
                            } else {
                                bottomSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                                listener.showBottom();
                            }
                        }
                        break;
                    default:
                        break;
                }
            }

            @Override
            public void onSlide(@NonNull View view, float v) {
                offset[0] = v;
            }
        });
        ivBack.setOnClickListener(v -> {
            SoftInputUtils.hideSoft((Activity) context);
            etSearchMap.clearFocus();
            hideSheet();
        });
    }

    public void showSheet(String content) {
        if (bottomSheetBehavior.getState() == BottomSheetBehavior.STATE_COLLAPSED) {
            bottomSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);
        }
        SoftInputUtils.showSoft((Activity) context, etSearchMap);
        etSearchMap.setText(content == null ? "" : content);
    }

    public void hideSheet() {
        if (bottomSheetBehavior.getState() == BottomSheetBehavior.STATE_EXPANDED) {
            bottomSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
        }
    }

    @OnClick({R.id.ll_delete_map})
    public void onClick(View view) {
        etSearchMap.setText("");
    }

    /**
     * 滑动到底部监听
     */
    public interface onBottomListener {
        void showBottom();
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count, int after) {

    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {
        String newText = s.toString().trim();
        InputtipsQuery inputquery = new InputtipsQuery(newText, "");
        inputquery.setCityLimit(true);
        Inputtips inputTips = new Inputtips(context, inputquery);
        inputTips.setInputtipsListener(this);
        inputTips.requestInputtipsAsyn();
    }

    @Override
    public void afterTextChanged(Editable s) {
        if (!TextUtils.isEmpty(etSearchMap.getText())) {
            llDeleteMap.setVisibility(VISIBLE);
        } else {
            llDeleteMap.setVisibility(GONE);
        }
    }

    @Override
    public void onGetInputtips(List<Tip> list, int i) {
        textInputListener.finishText(list);
    }

    /**
     * 关键字搜索
     */
    public interface textInputListener {
        void finishText(List<Tip> list);
    }
}
