package com.tofly.yxpc.widget;

import android.app.AlertDialog;
import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;

import androidx.appcompat.widget.DialogTitle;

import com.tofly.latte_core.utils.ToastUtils;
import com.tofly.yxpc.R;
import com.tofly.yxpc.utils.CheckPhoneUtils;

import java.util.Objects;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;


/**
 * @author ychk
 */
public class UserInfoDialog extends AlertDialog {

    @BindView(R.id.alertTitle)
    DialogTitle alertTitle;
    @BindView(R.id.edit_input1)
    EditText editInput1;
    @BindView(R.id.layout_input1)
    LinearLayout layoutInput1;
    @BindView(R.id.edit_input2)
    EditText editInput2;
    @BindView(R.id.layout_input2)
    LinearLayout layoutInput2;
    @BindView(R.id.edit_input3)
    EditText editInput3;
    @BindView(R.id.layout_input3)
    LinearLayout layoutInput3;
    @BindView(android.R.id.button2)
    Button button2;
    @BindView(android.R.id.button1)
    Button button1;

    private String mUserName;
    private String mUserPhone;
    private String email;
    private ClickListener mClickListener;
    private int inputType;

    public static final int INPUT_NAME = 0;
    public static final int INPUT_MOBILE = 1;
    public static final int INPUT_EMAIL = 2;

    public UserInfoDialog(Context context, String username, String mobile, String email, int inputType) {
        super(context);
        mUserName = username;
        mUserPhone = mobile;
        this.email = email;
        this.inputType = inputType;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Objects.requireNonNull(getWindow()).clearFlags(WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM);
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);
        setContentView(R.layout.dialog_user_info);
        ButterKnife.bind(this);
        initView();
    }


    public interface ClickListener {
        void doOk(String name, String mobile, String email);

        void doCancel();
    }

    public void setClickListener(ClickListener clicklistener) {
        this.mClickListener = clicklistener;
    }

    private void initView() {
        alertTitle.setText("修改用户信息");
        layoutInput1.setVisibility(View.GONE);
        layoutInput2.setVisibility(View.GONE);
        layoutInput3.setVisibility(View.GONE);
        switch (inputType) {
            case INPUT_NAME:
                layoutInput1.setVisibility(View.VISIBLE);
                if (!mUserName.isEmpty()) {
                    editInput1.setText(mUserName);
                }
                break;
            case INPUT_MOBILE:
                layoutInput2.setVisibility(View.VISIBLE);
                if ((!mUserPhone.isEmpty())) {
                    editInput2.setText(mUserPhone);
                }
                break;
            case INPUT_EMAIL:
                layoutInput3.setVisibility(View.VISIBLE);
                if ((!email.isEmpty())) {
                    editInput3.setText(email);
                }
                break;
            default:
                break;
        }
        button1.setText(android.R.string.ok);
        button2.setText(android.R.string.cancel);
    }

    @OnClick({android.R.id.button1, android.R.id.button2})
    public void onClick(View view) {
        switch (view.getId()) {
            /*Button OK*/
            case android.R.id.button1:
                String name = editInput1.getText().toString();
                if (inputType == INPUT_NAME) {
                    if (name.isEmpty()) {
                        ToastUtils.showToast("输入信息不能为空");
                        return;
                    }
                }

                String mobile = editInput2.getText().toString();
                if (inputType == INPUT_MOBILE) {
                    if (mobile.isEmpty()) {
                        ToastUtils.showToast("输入信息不能为空");
                        return;
                    }

                    if (!CheckPhoneUtils.isPhoneNum(mobile)) {
                        ToastUtils.showToast("请输入正确的手机号");
                        return;
                    }
                }
                String email = editInput3.getText().toString();
                if (inputType == INPUT_EMAIL) {
                    if (email.isEmpty()) {
                        ToastUtils.showToast("输入信息不能为空");
                        return;
                    }

                    if (!CheckPhoneUtils.isEmail(email)) {
                        ToastUtils.showToast("请输入正确的邮箱");
                        return;
                    }
                }
                if (mClickListener != null) {
                    mClickListener.doOk(name, mobile, email);
                }
                Objects.requireNonNull(getWindow()).setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);
                this.dismiss();
                break;
            case android.R.id.button2:
                if (mClickListener != null) {
                    mClickListener.doCancel();
                }
                Objects.requireNonNull(getWindow()).setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);
                this.dismiss();
                break;
            default:
                break;
        }
    }
}
