package com.tofly.latte_core.base;

import java.lang.ref.WeakReference;

/**
 * @author ychk
 * 控制器基类：
 * Presenter生命周期包装、View的绑定和解除，P层实现的基类
 */
public class BasePresenter<V> implements IPresenter<V> {

    private WeakReference<V> viewRef;

    public V getView() {
        return viewRef == null ? null : viewRef.get();
    }

    public boolean isViewAttached() {
        return viewRef != null && viewRef.get() != null;
    }


    @Override
    public void onAttachView(V view) {
        viewRef = new WeakReference<>(view);
    }

    @Override
    public void onDetachView() {
        detach();
    }

    private void detach() {
        if (viewRef != null) {
            viewRef.clear();
            viewRef = null;
        }
    }
}
