package com.tofly.latte_core.net.observer;

import com.tofly.latte_core.net.exception.ExceptionHandle;

import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;

/**
 * rxJava观察者封装
 *
 * @author ychk
 */
public abstract class BaseObserver<T> implements Observer<T> {

    private Disposable disposable;

    @Override
    public void onSubscribe(Disposable d) {
        disposable = d;
    }

    @Override
    public void onNext(T response) {
        onSuccess(response);
    }

    @Override
    public void onError(Throwable e) {
        onFailure(e, ExceptionHandle.handleException(e).getMessage());
    }

    @Override
    public void onComplete() {
        cancelRequest();
    }

    public abstract void onSuccess(T demo);

    public abstract void onFailure(Throwable e, String errorMsg);

    /**
     * 取消订阅
     */
    public void cancelRequest() {
        if (disposable != null && !disposable.isDisposed()) {
            disposable.dispose();
        }
    }

}
