package com.tofly.latte_core.utils;

import android.content.Context;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.LinearLayout;

import androidx.appcompat.app.AppCompatDialog;

import com.tofly.latte_core.R;
import com.wang.avi.AVLoadingIndicatorView;

/**
 * 加载工具类
 *
 * @author ychk
 */
public class LoaderUtils {

    private AppCompatDialog mLoadingDialog;

    private static class Holder {
        private static final LoaderUtils LOADER_UTILS = new LoaderUtils();
    }

    public static LoaderUtils getInstance() {
        return Holder.LOADER_UTILS;
    }

    public void showDialog(Context context) {
        if (context != null) {
            View view = LayoutInflater.from(context).inflate(R.layout.item_loader_dialog, null);
            final AVLoadingIndicatorView avLoadingIndicatorView = view.findViewById(R.id.av_loader_dialog);
            avLoadingIndicatorView.setIndicator(LoaderStyle.BallClipRotatePulseIndicator.name());
            if (mLoadingDialog == null) {
                mLoadingDialog = new AppCompatDialog(context, R.style.dialog);
                mLoadingDialog.setCancelable(true);
                mLoadingDialog.setCanceledOnTouchOutside(true);
                mLoadingDialog.setContentView(view, new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,
                        LinearLayout.LayoutParams.MATCH_PARENT));
                mLoadingDialog.show();
                avLoadingIndicatorView.smoothToShow();
                mLoadingDialog.setOnKeyListener((dialogInterface, keyCode, keyEvent) -> {
                    if (keyCode == KeyEvent.KEYCODE_BACK) {
                        if (mLoadingDialog != null) {
                            mLoadingDialog.dismiss();
                        }
                        avLoadingIndicatorView.hide();
                        return true;
                    }
                    return false;
                });
            } else if (!mLoadingDialog.isShowing()) {
                mLoadingDialog.show();
            }
        }
    }

    public void stopDialog() {
        if (mLoadingDialog != null) {
            mLoadingDialog.dismiss();
            mLoadingDialog = null;
        }
    }
}
